@@ -38,6 +38,7 @@ Torus Knot Software Ltd.
 #include "OgreVector3.h"
 #include "OgreHardwareBufferManager.h"
 #include "OgreMesh.h"
+#include "OgreRenderable.h"
 
 namespace Ogre {
 	/** Defines an instance of a discrete, movable object based on a Mesh.
@@ -86,7 +87,7 @@ namespace Ogre {
 		Entity();
 		/** Private constructor - specify name (the usual constructor used).
 		*/
-		Entity( const String& name, MeshPtr& mesh);
+		Entity( const String& name, const MeshPtr& mesh);
 
 		/** The Mesh that this Entity is based on.
 		*/
@@ -224,6 +225,9 @@ namespace Ogre {
 		/// Last parent xform
 		Matrix4 mLastParentXform;
 
+		/// Mesh state count, used to detect differences
+		size_t mMeshStateCount;
+
 		/** Builds a list of SubEntities based on the SubMeshes contained in the Mesh. */
 		void buildSubEntityList(MeshPtr& mesh, SubEntityList* sublist);
 
@@ -259,8 +263,6 @@ namespace Ogre {
 		/// Bounding box that 'contains' all the mesh of each child entity
 		mutable AxisAlignedBox mFullBoundingBox;
 
-		bool mNormaliseNormals;
-
 		ShadowRenderableList mShadowRenderables;
 
 		/** Nested class to allow entity shadows. */
@@ -287,10 +289,6 @@ namespace Ogre {
 			~EntityShadowRenderable();
 			/// Overridden from ShadowRenderable
 			void getWorldTransforms(Matrix4* xform) const;
-			/// Overridden from ShadowRenderable
-			const Quaternion& getWorldOrientation(void) const;
-			/// Overridden from ShadowRenderable
-			const Vector3& getWorldPosition(void) const;
 			HardwareVertexBufferSharedPtr getPositionBuffer(void) { return mPositionBuffer; }
 			HardwareVertexBufferSharedPtr getWBuffer(void) { return mWBuffer; }
 			/// Rebind the source positions (for temp buffer users)
@@ -342,6 +340,17 @@ namespace Ogre {
 		*/
 		void setMaterialName(const String& name);
 
+		
+		/** Sets the material to use for the whole of this entity.
+			@remarks
+				This is a shortcut method to set all the materials for all
+				subentities of this entity. Only use this method is you want to
+				set the same material for all subentities or if you know there
+				is only one. Otherwise call getSubEntity() and call the same
+				method on the individual SubEntity.
+		*/
+		void setMaterial(const MaterialPtr& material);
+
 		/** Overridden - see MovableObject.
 		*/
 		void _notifyCurrentCamera(Camera* cam);
@@ -404,6 +413,10 @@ namespace Ogre {
         */
         size_t getNumManualLodLevels(void) const;
 
+		/** Returns the current LOD used to render
+		*/
+		ushort getCurrentLodIndex() { return mMeshLodIndex; }
+
 		/** Sets a level-of-detail bias for the mesh detail of this entity.
 		@remarks
 		Level of detail reduction is normally applied automatically based on the Mesh
@@ -511,20 +524,6 @@ namespace Ogre {
 		/** @copy MovableObject::getWorldBoundingSphere */
 		const Sphere& getWorldBoundingSphere(bool derive = false) const;
 
-        /** If set to true, this forces normals of this entity to be normalised
-            dynamically by the hardware.
-        @remarks
-            This option can be used to prevent lighting variations when scaling an
-            Entity using a SceneNode - normally because this scaling is hardware
-            based, the normals get scaled too which causes lighting to become inconsistent.
-            However, this has an overhead so only do this if you really need to.
-        */
-        void setNormaliseNormals(bool normalise) { mNormaliseNormals = normalise; }
-
-        /** Returns true if this entity has auto-normalisation of normals set. */
-        bool getNormaliseNormals(void) const {return mNormaliseNormals; }
-
-
         /** Overridden member from ShadowCaster. */
         EdgeData* getEdgeList(void);
 		/** Overridden member from ShadowCaster. */
@@ -645,7 +644,7 @@ namespace Ogre {
 
 		/** Advanced method to perform all the updates required for an animated entity.
 		@remarks
-		You don't normally need to call this, but it's here incase you wish
+		You don't normally need to call this, but it's here in case you wish
 		to manually update the animation of an Entity at a specific point in
 		time. Animation will not be updated more than once a frame no matter
 		how many times you call this method.
@@ -746,6 +745,11 @@ namespace Ogre {
 		*/
 		void backgroundLoadingComplete(Resource* res);
 
+		/// @copydoc MovableObject::visitRenderables
+		void visitRenderables(Renderable::Visitor* visitor, 
+			bool debugRenderables = false);
+
+
 
 
 	};
