@@ -57,12 +57,14 @@ namespace Ogre {
          mInverseViewMatrixDirty(true),
          mInverseTransposeWorldMatrixDirty(true),
          mInverseTransposeWorldViewMatrixDirty(true),
+		 mCameraPositionDirty(true),
          mCameraPositionObjectSpaceDirty(true),
-         mCameraPositionDirty(true),
 		 mSceneDepthRangeDirty(true),
-		 mShadowCamDepthRangesDirty(true),
+		 mLodCameraPositionDirty(true),
+		 mLodCameraPositionObjectSpaceDirty(true),
          mCurrentRenderable(0),
          mCurrentCamera(0), 
+		 mCameraRelativeRendering(false),
          mCurrentRenderTarget(0),
          mCurrentViewport(0), 
 		 mCurrentSceneManager(0),
@@ -75,13 +77,30 @@ namespace Ogre {
 		for(size_t i = 0; i < OGRE_MAX_SIMULTANEOUS_LIGHTS; ++i)
 		{
 			mTextureViewProjMatrixDirty[i] = true;
+			mTextureWorldViewProjMatrixDirty[i] = true;
+			mSpotlightViewProjMatrixDirty[i] = true;
+			mSpotlightWorldViewProjMatrixDirty[i] = true;
 			mCurrentTextureProjector[i] = 0;
+			mShadowCamDepthRangesDirty[i] = false;
 		}
 
     }
     //-----------------------------------------------------------------------------
     AutoParamDataSource::~AutoParamDataSource()
     {
+    }
+	//-----------------------------------------------------------------------------
+    const Light& AutoParamDataSource::getLight(size_t index) const
+    {
+        // If outside light range, return a blank light to ensure zeroised for program
+		if (index < mCurrentLightList->size())
+		{
+			return *((*mCurrentLightList)[index]);
+		}
+		else
+        {
+            return mBlankLight;
+        }        
     }
     //-----------------------------------------------------------------------------
     void AutoParamDataSource::setCurrentRenderable(const Renderable* rend)
@@ -99,11 +118,20 @@ namespace Ogre {
 		mInverseTransposeWorldMatrixDirty = true;
 		mInverseTransposeWorldViewMatrixDirty = true;
 		mCameraPositionObjectSpaceDirty = true;
+		mLodCameraPositionObjectSpaceDirty = true;
+		for(size_t i = 0; i < OGRE_MAX_SIMULTANEOUS_LIGHTS; ++i)
+		{
+			mTextureWorldViewProjMatrixDirty[i] = true;
+			mSpotlightWorldViewProjMatrixDirty[i] = true;
+		}
+
     }
     //-----------------------------------------------------------------------------
-    void AutoParamDataSource::setCurrentCamera(const Camera* cam)
+    void AutoParamDataSource::setCurrentCamera(const Camera* cam, bool useCameraRelative)
     {
         mCurrentCamera = cam;
+		mCameraRelativeRendering = useCameraRelative;
+		mCameraRelativePosition = cam->getDerivedPosition();
         mViewMatrixDirty = true;
         mProjMatrixDirty = true;
         mWorldViewMatrixDirty = true;
@@ -114,13 +142,113 @@ namespace Ogre {
         mInverseTransposeWorldViewMatrixDirty = true;
         mCameraPositionObjectSpaceDirty = true;
         mCameraPositionDirty = true;
+		mLodCameraPositionObjectSpaceDirty = true;
+		mLodCameraPositionDirty = true;
     }
     //-----------------------------------------------------------------------------
     void AutoParamDataSource::setCurrentLightList(const LightList* ll)
     {
         mCurrentLightList = ll;
-		mShadowCamDepthRangesDirty = true;
+		for(size_t i = 0; i < ll->size() && i < OGRE_MAX_SIMULTANEOUS_LIGHTS; ++i)
+		{
+			mSpotlightViewProjMatrixDirty[i] = true;
+			mSpotlightWorldViewProjMatrixDirty[i] = true;
+		}
+
     }
+	//---------------------------------------------------------------------
+	float AutoParamDataSource::getLightNumber(size_t index) const
+	{
+		return static_cast<float>(getLight(index)._getIndexInFrame());
+	}
+	//-----------------------------------------------------------------------------
+	const ColourValue& AutoParamDataSource::getLightDiffuseColour(size_t index) const
+	{
+		return getLight(index).getDiffuseColour();
+	}
+	//-----------------------------------------------------------------------------
+	const ColourValue& AutoParamDataSource::getLightSpecularColour(size_t index) const
+	{
+		return getLight(index).getSpecularColour();
+	}
+	//-----------------------------------------------------------------------------
+	const ColourValue AutoParamDataSource::getLightDiffuseColourWithPower(size_t index) const
+	{
+		const Light& l = getLight(index);
+		ColourValue scaled(l.getDiffuseColour());
+		Real power = l.getPowerScale();
+		// scale, but not alpha
+		scaled.r *= power;
+		scaled.g *= power;
+		scaled.b *= power;
+		return scaled;
+	}
+	//-----------------------------------------------------------------------------
+	const ColourValue AutoParamDataSource::getLightSpecularColourWithPower(size_t index) const
+	{
+		const Light& l = getLight(index);
+		ColourValue scaled(l.getSpecularColour());
+		Real power = l.getPowerScale();
+		// scale, but not alpha
+		scaled.r *= power;
+		scaled.g *= power;
+		scaled.b *= power;
+		return scaled;
+	}
+	//-----------------------------------------------------------------------------
+	const Vector3& AutoParamDataSource::getLightPosition(size_t index) const
+	{
+		return getLight(index).getDerivedPosition(true);
+	}
+	//-----------------------------------------------------------------------------
+	Vector4 AutoParamDataSource::getLightAs4DVector(size_t index) const
+	{
+		return getLight(index).getAs4DVector(true);
+	}
+	//-----------------------------------------------------------------------------
+	const Vector3& AutoParamDataSource::getLightDirection(size_t index) const
+	{
+		return getLight(index).getDerivedDirection();
+	}
+	//-----------------------------------------------------------------------------
+	Real AutoParamDataSource::getLightPowerScale(size_t index) const
+	{
+		return getLight(index).getPowerScale();
+	}
+	//-----------------------------------------------------------------------------
+	Vector4 AutoParamDataSource::getLightAttenuation(size_t index) const
+	{
+		// range, const, linear, quad
+        const Light& l = getLight(index);
+		return Vector4(l.getAttenuationRange(),
+				       l.getAttenuationConstant(),
+					   l.getAttenuationLinear(),
+					   l.getAttenuationQuadric());
+	}
+	//-----------------------------------------------------------------------------
+	Vector4 AutoParamDataSource::getSpotlightParams(size_t index) const
+	{
+		// inner, outer, fallof, isSpot
+		const Light& l = getLight(index);
+		if (l.getType() == Light::LT_SPOTLIGHT)
+		{
+			return Vector4(Math::Cos(l.getSpotlightInnerAngle().valueRadians() * 0.5),
+						   Math::Cos(l.getSpotlightOuterAngle().valueRadians() * 0.5),
+						   l.getSpotlightFalloff(),
+						   1.0);
+		}
+		else
+		{
+			// Use safe values which result in no change to point & dir light calcs
+			// The spot factor applied to the usual lighting calc is 
+			// pow((dot(spotDir, lightDir) - y) / (x - y), z)
+			// Therefore if we set z to 0.0f then the factor will always be 1
+			// since pow(anything, 0) == 1
+			// However we also need to ensure we don't overflow because of the division
+			// therefore set x = 1 and y = 0 so divisor doesn't change scale
+			return Vector4(1.0, 0.0, 0.0, 1.0); // since the main op is pow(.., vec4.z), this will result in 1.0
+		}
+	}
 	//-----------------------------------------------------------------------------
 	void AutoParamDataSource::setMainCamBoundsInfo(VisibleObjectsBoundsInfo* info)
 	{
@@ -147,6 +275,13 @@ namespace Ogre {
             mWorldMatrixArray = mWorldMatrix;
             mCurrentRenderable->getWorldTransforms(mWorldMatrix);
             mWorldMatrixCount = mCurrentRenderable->getNumWorldTransforms();
+			if (mCameraRelativeRendering)
+			{
+				for (size_t i = 0; i < mWorldMatrixCount; ++i)
+				{
+					mWorldMatrix[i].setTrans(mWorldMatrix[i].getTrans() - mCameraRelativePosition);
+				}
+			}
             mWorldMatrixDirty = false;
         }
         return mWorldMatrixArray[0];
@@ -154,25 +289,15 @@ namespace Ogre {
     //-----------------------------------------------------------------------------
     size_t AutoParamDataSource::getWorldMatrixCount(void) const
     {
-        if (mWorldMatrixDirty)
-        {
-            mWorldMatrixArray = mWorldMatrix;
-            mCurrentRenderable->getWorldTransforms(mWorldMatrix);
-            mWorldMatrixCount = mCurrentRenderable->getNumWorldTransforms();
-            mWorldMatrixDirty = false;
-        }
+		// trigger derivation
+		getWorldMatrix();
         return mWorldMatrixCount;
     }
     //-----------------------------------------------------------------------------
     const Matrix4* AutoParamDataSource::getWorldMatrixArray(void) const
     {
-        if (mWorldMatrixDirty)
-        {
-            mWorldMatrixArray = mWorldMatrix;
-            mCurrentRenderable->getWorldTransforms(mWorldMatrix);
-            mWorldMatrixCount = mCurrentRenderable->getNumWorldTransforms();
-            mWorldMatrixDirty = false;
-        }
+		// trigger derivation
+		getWorldMatrix();
         return mWorldMatrixArray;
     }
     //-----------------------------------------------------------------------------
@@ -183,7 +308,14 @@ namespace Ogre {
             if (mCurrentRenderable && mCurrentRenderable->getUseIdentityView())
                 mViewMatrix = Matrix4::IDENTITY;
             else
+			{
                 mViewMatrix = mCurrentCamera->getViewMatrix(true);
+				if (mCameraRelativeRendering)
+				{
+					mViewMatrix.setTrans(Vector3::ZERO);
+				}
+
+			}
             mViewMatrixDirty = false;
         }
         return mViewMatrix;
@@ -323,24 +455,46 @@ namespace Ogre {
         }
         return mCameraPositionObjectSpace;
     }
-    //-----------------------------------------------------------------------------
-    const Light& AutoParamDataSource::getLight(size_t index) const
-    {
-        // If outside light range, return a blank light to ensure zeroised for program
-        if (mCurrentLightList->size() <= index)
-        {
-            return mBlankLight;
-        }
-        else
-        {
-            return *((*mCurrentLightList)[index]);
-        }
-    }
+	//-----------------------------------------------------------------------------
+	const Vector4& AutoParamDataSource::getLodCameraPosition(void) const
+	{
+		if(mLodCameraPositionDirty)
+		{
+			Vector3 vec3 = mCurrentCamera->getLodCamera()->getDerivedPosition();
+			mLodCameraPosition[0] = vec3[0];
+			mLodCameraPosition[1] = vec3[1];
+			mLodCameraPosition[2] = vec3[2];
+			mLodCameraPosition[3] = 1.0;
+			mLodCameraPositionDirty = false;
+		}
+		return mLodCameraPosition;
+	}
+	//-----------------------------------------------------------------------------
+	const Vector4& AutoParamDataSource::getLodCameraPositionObjectSpace(void) const
+	{
+		if (mLodCameraPositionObjectSpaceDirty)
+		{
+			mLodCameraPositionObjectSpace = 
+				getInverseWorldMatrix().transformAffine(mCurrentCamera->getLodCamera()->getDerivedPosition());
+			mLodCameraPositionObjectSpaceDirty = false;
+		}
+		return mLodCameraPositionObjectSpace;
+	}
     //-----------------------------------------------------------------------------
 	void AutoParamDataSource::setAmbientLightColour(const ColourValue& ambient)
 	{
 		mAmbientLight = ambient;
 	}
+	//---------------------------------------------------------------------
+	float AutoParamDataSource::getLightCount() const
+	{
+		return static_cast<float>(mCurrentLightList->size());
+	}
+	//---------------------------------------------------------------------
+	int AutoParamDataSource::getLightCastsShadows(size_t index) const
+	{
+		return static_cast<int>(getLight(index).getCastShadows() ? 1 : 0);
+	}
     //-----------------------------------------------------------------------------
 	const ColourValue& AutoParamDataSource::getAmbientLightColour(void) const
 	{
@@ -449,23 +603,158 @@ namespace Ogre {
     //-----------------------------------------------------------------------------
     void AutoParamDataSource::setTextureProjector(const Frustum* frust, size_t index = 0)
     {
-        mCurrentTextureProjector[index] = frust;
-        mTextureViewProjMatrixDirty[index] = true;
+		if (index < OGRE_MAX_SIMULTANEOUS_LIGHTS)
+		{
+			mCurrentTextureProjector[index] = frust;
+			mTextureViewProjMatrixDirty[index] = true;
+			mTextureWorldViewProjMatrixDirty[index] = true;
+			mShadowCamDepthRangesDirty[index] = true;
+		}
 
     }
     //-----------------------------------------------------------------------------
     const Matrix4& AutoParamDataSource::getTextureViewProjMatrix(size_t index) const
     {
-        if (mTextureViewProjMatrixDirty[index] && mCurrentTextureProjector[index])
-        {
-            mTextureViewProjMatrix[index] = 
-                PROJECTIONCLIPSPACE2DTOIMAGESPACE_PERSPECTIVE * 
-                mCurrentTextureProjector[index]->getProjectionMatrixWithRSDepth() * 
-				mCurrentTextureProjector[index]->getViewMatrix();
-            mTextureViewProjMatrixDirty[index] = false;
-        }
-        return mTextureViewProjMatrix[index];
+		if (index < OGRE_MAX_SIMULTANEOUS_LIGHTS)
+		{
+			if (mTextureViewProjMatrixDirty[index] && mCurrentTextureProjector[index])
+			{
+				if (mCameraRelativeRendering)
+				{
+					// World positions are now going to be relative to the camera position
+					// so we need to alter the projector view matrix to compensate
+					Matrix4 viewMatrix;
+					mCurrentTextureProjector[index]->calcViewMatrixRelative(
+						mCurrentCamera->getDerivedPosition(), viewMatrix);
+					mTextureViewProjMatrix[index] = 
+						PROJECTIONCLIPSPACE2DTOIMAGESPACE_PERSPECTIVE * 
+						mCurrentTextureProjector[index]->getProjectionMatrixWithRSDepth() * 
+						viewMatrix;
+				}
+				else
+				{
+					mTextureViewProjMatrix[index] = 
+						PROJECTIONCLIPSPACE2DTOIMAGESPACE_PERSPECTIVE * 
+						mCurrentTextureProjector[index]->getProjectionMatrixWithRSDepth() * 
+						mCurrentTextureProjector[index]->getViewMatrix();
+				}
+				mTextureViewProjMatrixDirty[index] = false;
+			}
+			return mTextureViewProjMatrix[index];
+		}
+		else
+			return Matrix4::IDENTITY;
     }
+	//-----------------------------------------------------------------------------
+	const Matrix4& AutoParamDataSource::getTextureWorldViewProjMatrix(size_t index) const
+	{
+		if (index < OGRE_MAX_SIMULTANEOUS_LIGHTS)
+		{
+			if (mTextureWorldViewProjMatrixDirty[index] && mCurrentTextureProjector[index])
+			{
+				mTextureWorldViewProjMatrix[index] = 
+					getTextureViewProjMatrix(index)	* getWorldMatrix();
+				mTextureWorldViewProjMatrixDirty[index] = false;
+			}
+			return mTextureWorldViewProjMatrix[index];
+		}
+		else
+			return Matrix4::IDENTITY;
+	}
+	//-----------------------------------------------------------------------------
+	const Matrix4& AutoParamDataSource::getSpotlightViewProjMatrix(size_t index) const
+	{
+		if (index < OGRE_MAX_SIMULTANEOUS_LIGHTS)
+		{
+			const Light& l = getLight(index);
+
+			if (&l != &mBlankLight && 
+				l.getType() == Light::LT_SPOTLIGHT &&
+				mSpotlightViewProjMatrixDirty[index])
+			{
+				Frustum frust;
+				SceneNode dummyNode(0);
+				dummyNode.attachObject(&frust);
+
+				frust.setProjectionType(PT_PERSPECTIVE);
+				frust.setFOVy(l.getSpotlightOuterAngle());
+				frust.setAspectRatio(1.0f);
+				// set near clip the same as main camera, since they are likely
+				// to both reflect the nature of the scene
+				frust.setNearClipDistance(mCurrentCamera->getNearClipDistance());
+				// Calculate position, which same as spotlight position, in camera-relative coords if required
+				dummyNode.setPosition(l.getDerivedPosition(true));
+				// Calculate direction, which same as spotlight direction
+				Vector3 dir = - l.getDerivedDirection(); // backwards since point down -z
+				dir.normalise();
+				Vector3 up = Vector3::UNIT_Y;
+				// Check it's not coincident with dir
+				if (Math::Abs(up.dotProduct(dir)) >= 1.0f)
+				{
+					// Use camera up
+					up = Vector3::UNIT_Z;
+				}
+				// cross twice to rederive, only direction is unaltered
+				Vector3 left = dir.crossProduct(up);
+				left.normalise();
+				up = dir.crossProduct(left);
+				up.normalise();
+				// Derive quaternion from axes
+				Quaternion q;
+				q.FromAxes(left, up, dir);
+				dummyNode.setOrientation(q);
+
+				// The view matrix here already includes camera-relative changes if necessary
+				// since they are built into the frustum position
+				mSpotlightViewProjMatrix[index] = 
+					PROJECTIONCLIPSPACE2DTOIMAGESPACE_PERSPECTIVE * 
+					frust.getProjectionMatrixWithRSDepth() * 
+					frust.getViewMatrix();
+
+				mSpotlightViewProjMatrixDirty[index] = false;
+			}
+			return mSpotlightViewProjMatrix[index];
+		}
+		else
+			return Matrix4::IDENTITY;
+	}
+	//-----------------------------------------------------------------------------
+	const Matrix4& AutoParamDataSource::getSpotlightWorldViewProjMatrix(size_t index) const
+	{
+		if (index < OGRE_MAX_SIMULTANEOUS_LIGHTS)
+		{
+			const Light& l = getLight(index);
+
+			if (&l != &mBlankLight && 
+				l.getType() == Light::LT_SPOTLIGHT &&
+				mSpotlightWorldViewProjMatrixDirty[index])
+			{
+				mSpotlightWorldViewProjMatrix[index] = 
+					getSpotlightViewProjMatrix(index) * getWorldMatrix();
+				mSpotlightWorldViewProjMatrixDirty[index] = false;
+			}
+			return mSpotlightWorldViewProjMatrix[index];
+		}
+		else
+			return Matrix4::IDENTITY;
+	}
+//-----------------------------------------------------------------------------
+  const Matrix4& AutoParamDataSource::getTextureTransformMatrix(size_t index) const
+  {
+    // make sure the current pass is set
+    assert(mCurrentPass && "current pass is NULL!");
+    // check if there is a texture unit with the given index in the current pass
+    if(index < mCurrentPass->getNumTextureUnitStates())
+    {
+      // texture unit existent, return its currently set transform
+      return mCurrentPass->getTextureUnitState(static_cast<unsigned short>(index))->getTextureTransform();
+    }
+    else
+    {
+      // no such texture unit, return unity
+      return Matrix4::IDENTITY;
+    }
+  }
     //-----------------------------------------------------------------------------
     void AutoParamDataSource::setCurrentRenderTarget(const RenderTarget* target)
     {
@@ -499,7 +788,7 @@ namespace Ogre {
 		{
 			// Calculate based on object space light distance
 			// compared to light attenuation range
-			Vector3 objPos = getInverseWorldMatrix().transformAffine(l.getDerivedPosition());
+			Vector3 objPos = getInverseWorldMatrix().transformAffine(l.getDerivedPosition(true));
 			return l.getAttenuationRange() - objPos.length();
 		}
 	}
@@ -748,45 +1037,46 @@ namespace Ogre {
 
 	}
 	//-----------------------------------------------------------------------------
-	const Vector4& AutoParamDataSource::getShadowSceneDepthRange(size_t lightIndex) const
+	const Vector4& AutoParamDataSource::getShadowSceneDepthRange(size_t index) const
 	{
 		static Vector4 dummy(0, 100000, 100000, 1/100000);
 
 		if (!mCurrentSceneManager->isShadowTechniqueTextureBased())
 			return dummy;
 
-		if (mShadowCamDepthRangesDirty)
+		if (index < OGRE_MAX_SIMULTANEOUS_LIGHTS)
 		{
-			mShadowCamDepthRanges.clear();
-			for (LightList::const_iterator i = mCurrentLightList->begin();
-				i != mCurrentLightList->end(); ++i)
+			if (mShadowCamDepthRangesDirty[index] && mCurrentTextureProjector[index])
 			{
-				// Skip non shadow casting lights
-				if (!(*i)->getCastShadows())
-					continue;
-
 				const VisibleObjectsBoundsInfo& info = 
-					mCurrentSceneManager->getShadowCasterBoundsInfo(*i);
+					mCurrentSceneManager->getVisibleObjectsBoundsInfo(
+						(Camera*)mCurrentTextureProjector[index]);
 
-				mShadowCamDepthRanges.push_back(Vector4(
-					info.minDistance, 
-					info.maxDistance, 
-					info.maxDistance - info.minDistance,
-					1.0f / (info.maxDistance - info.minDistance)));
+				Real depthRange = info.maxDistance - info.minDistance;
+				if (depthRange > std::numeric_limits<Real>::epsilon())
+				{
+					mShadowCamDepthRanges[index] = Vector4(
+						info.minDistance,
+						info.maxDistance,
+						depthRange,
+						1.0f / depthRange);
+				}
+				else
+				{
+					mShadowCamDepthRanges[index] = dummy;
+				}
+
+				mShadowCamDepthRangesDirty[index] = false;
 			}
-
-			mShadowCamDepthRangesDirty = false;
-		}
-
-		if (lightIndex >= mShadowCamDepthRanges.size())
-		{
-			return dummy;
+			return mShadowCamDepthRanges[index];
 		}
 		else
-		{
-			return mShadowCamDepthRanges[lightIndex];
-		}
-
+			return dummy;
+	}
+	//---------------------------------------------------------------------
+	const ColourValue& AutoParamDataSource::getShadowColour() const
+	{
+		return mCurrentSceneManager->getShadowColour();
 	}
 
 }
