@@ -173,14 +173,14 @@ namespace Ogre {
     String DataStream::getAsString(void)
     {
         // Read the entire buffer
-        char* pBuf = new char[mSize+1];
+        char* pBuf = OGRE_ALLOC_T(char, mSize+1, MEMCATEGORY_GENERAL);
         // Ensure read from begin of stream
         seek(0);
         read(pBuf, mSize);
         pBuf[mSize] = '\0';
         String str;
         str.insert(0, pBuf, mSize);
-        delete [] pBuf;
+        OGRE_FREE(pBuf, MEMCATEGORY_GENERAL);
         return str;
     }
     //-----------------------------------------------------------------------
@@ -192,6 +192,7 @@ namespace Ogre {
         mSize = size;
         mEnd = mData + mSize;
         mFreeOnClose = freeOnClose;
+        assert(mEnd >= mPos);
     }
     //-----------------------------------------------------------------------
     MemoryDataStream::MemoryDataStream(const String& name, void* pMem, size_t size, 
@@ -202,6 +203,7 @@ namespace Ogre {
         mSize = size;
         mEnd = mData + mSize;
         mFreeOnClose = freeOnClose;
+        assert(mEnd >= mPos);
     }
     //-----------------------------------------------------------------------
     MemoryDataStream::MemoryDataStream(DataStream& sourceStream, 
@@ -210,11 +212,11 @@ namespace Ogre {
     {
         // Copy data from incoming stream
         mSize = sourceStream.size();
-        mData = new uchar[mSize];
-        sourceStream.read(mData, mSize);
+        mData = OGRE_ALLOC_T(uchar, mSize, MEMCATEGORY_GENERAL);
         mPos = mData;
-        mEnd = mData + mSize;
+        mEnd = mData + sourceStream.read(mData, mSize);
         mFreeOnClose = freeOnClose;
+        assert(mEnd >= mPos);
     }
     //-----------------------------------------------------------------------
     MemoryDataStream::MemoryDataStream(DataStreamPtr& sourceStream, 
@@ -223,11 +225,11 @@ namespace Ogre {
     {
         // Copy data from incoming stream
         mSize = sourceStream->size();
-        mData = new uchar[mSize];
-        sourceStream->read(mData, mSize);
+        mData = OGRE_ALLOC_T(uchar, mSize, MEMCATEGORY_GENERAL);
         mPos = mData;
-        mEnd = mData + mSize;
+        mEnd = mData + sourceStream->read(mData, mSize);
         mFreeOnClose = freeOnClose;
+        assert(mEnd >= mPos);
     }
     //-----------------------------------------------------------------------
     MemoryDataStream::MemoryDataStream(const String& name, DataStream& sourceStream, 
@@ -236,11 +238,11 @@ namespace Ogre {
     {
         // Copy data from incoming stream
         mSize = sourceStream.size();
-        mData = new uchar[mSize];
-        sourceStream.read(mData, mSize);
+        mData = OGRE_ALLOC_T(uchar, mSize, MEMCATEGORY_GENERAL);
         mPos = mData;
-        mEnd = mData + mSize;
+        mEnd = mData + sourceStream.read(mData, mSize);
         mFreeOnClose = freeOnClose;
+        assert(mEnd >= mPos);
     }
     //-----------------------------------------------------------------------
     MemoryDataStream::MemoryDataStream(const String& name, const DataStreamPtr& sourceStream, 
@@ -249,11 +251,11 @@ namespace Ogre {
     {
         // Copy data from incoming stream
         mSize = sourceStream->size();
-        mData = new uchar[mSize];
-        sourceStream->read(mData, mSize);
+        mData = OGRE_ALLOC_T(uchar, mSize, MEMCATEGORY_GENERAL);
         mPos = mData;
-        mEnd = mData + mSize;
+        mEnd = mData + sourceStream->read(mData, mSize);
         mFreeOnClose = freeOnClose;
+        assert(mEnd >= mPos);
     }
     //-----------------------------------------------------------------------
     MemoryDataStream::MemoryDataStream(size_t size, bool freeOnClose)
@@ -261,9 +263,10 @@ namespace Ogre {
     {
         mSize = size;
         mFreeOnClose = freeOnClose;
-        mData = new uchar[size];
+        mData = OGRE_ALLOC_T(uchar, mSize, MEMCATEGORY_GENERAL);
         mPos = mData;
         mEnd = mData + mSize;
+        assert(mEnd >= mPos);
     }
     //-----------------------------------------------------------------------
     MemoryDataStream::MemoryDataStream(const String& name, size_t size, 
@@ -272,9 +275,10 @@ namespace Ogre {
     {
         mSize = size;
         mFreeOnClose = freeOnClose;
-        mData = new uchar[size];
+        mData = OGRE_ALLOC_T(uchar, mSize, MEMCATEGORY_GENERAL);
         mPos = mData;
         mEnd = mData + mSize;
+        assert(mEnd >= mPos);
     }
     //-----------------------------------------------------------------------
     MemoryDataStream::~MemoryDataStream()
@@ -291,6 +295,8 @@ namespace Ogre {
         if (cnt == 0)
             return 0;
 
+        assert (cnt<=count);
+
         memcpy(buf, mPos, cnt);
         mPos += cnt;
         return cnt;
@@ -382,7 +388,7 @@ namespace Ogre {
     {
         if (mFreeOnClose && mData)
         {
-            delete [] mData;
+            OGRE_FREE(mData, MEMCATEGORY_GENERAL);
             mData = 0;
         }
 
@@ -539,7 +545,7 @@ namespace Ogre {
             if (mFreeOnClose)
             {
                 // delete the stream too
-                delete mpStream;
+                OGRE_DELETE_T(mpStream, basic_ifstream, MEMCATEGORY_GENERAL);
                 mpStream = 0;
             }
         }
