@@ -68,7 +68,7 @@ namespace Ogre {
 		//Only delete if this was not a dynamic image (meaning app holds & destroys buffer)
 		if( m_pBuffer && m_bAutoDelete )
 		{
-			delete[] m_pBuffer;
+			OGRE_FREE(m_pBuffer, MEMCATEGORY_GENERAL);
 			m_pBuffer = NULL;
 		}
 	}
@@ -78,7 +78,7 @@ namespace Ogre {
 	{
 		if( m_pBuffer && m_bAutoDelete )
 		{
-			delete[] m_pBuffer;
+			OGRE_FREE(m_pBuffer, MEMCATEGORY_GENERAL);
 			m_pBuffer = NULL;
 		}
 		m_uWidth = img.m_uWidth;
@@ -93,7 +93,7 @@ namespace Ogre {
 		//Only create/copy when previous data was not dynamic data
 		if( m_bAutoDelete )
 		{
-			m_pBuffer = new uchar[ m_uSize ];
+			m_pBuffer = OGRE_ALLOC_T(uchar, m_uSize, MEMCATEGORY_GENERAL);
 			memcpy( m_pBuffer, img.m_pBuffer, m_uSize );
 		}
 		else
@@ -131,7 +131,7 @@ namespace Ogre {
 		switch (m_ucPixelSize)
 		{
 		case 1:
-			pTempBuffer1 = new uchar[m_uWidth * m_uHeight];
+			pTempBuffer1 = OGRE_ALLOC_T(uchar, m_uWidth * m_uHeight, MEMCATEGORY_GENERAL);
 			for (y = 0; y < m_uHeight; y++)
 			{
 				dst1 = (pTempBuffer1 + ((y * m_uWidth) + m_uWidth - 1));
@@ -140,11 +140,11 @@ namespace Ogre {
 			}
 
 			memcpy(m_pBuffer, pTempBuffer1, m_uWidth * m_uHeight * sizeof(uchar));
-			delete [] pTempBuffer1;
+			OGRE_FREE(pTempBuffer1, MEMCATEGORY_GENERAL);
 			break;
 
 		case 2:
-			pTempBuffer2 = new ushort[m_uWidth * m_uHeight];
+			pTempBuffer2 = OGRE_ALLOC_T(ushort, m_uWidth * m_uHeight, MEMCATEGORY_GENERAL);
 			for (y = 0; y < m_uHeight; y++)
 			{
 				dst2 = (pTempBuffer2 + ((y * m_uWidth) + m_uWidth - 1));
@@ -153,11 +153,11 @@ namespace Ogre {
 			}
 
 			memcpy(m_pBuffer, pTempBuffer2, m_uWidth * m_uHeight * sizeof(ushort));
-			delete [] pTempBuffer2;
+			OGRE_FREE(pTempBuffer2, MEMCATEGORY_GENERAL);
 			break;
 
 		case 3:
-			pTempBuffer3 = new uchar[m_uWidth * m_uHeight * 3];
+			pTempBuffer3 = OGRE_ALLOC_T(uchar, m_uWidth * m_uHeight * 3, MEMCATEGORY_GENERAL);
 			for (y = 0; y < m_uHeight; y++)
 			{
 				size_t offset = ((y * m_uWidth) + (m_uWidth - 1)) * 3;
@@ -171,11 +171,11 @@ namespace Ogre {
 			}
 
 			memcpy(m_pBuffer, pTempBuffer3, m_uWidth * m_uHeight * sizeof(uchar) * 3);
-			delete [] pTempBuffer3;
+			OGRE_FREE(pTempBuffer3, MEMCATEGORY_GENERAL);
 			break;
 
 		case 4:
-			pTempBuffer4 = new uint[m_uWidth * m_uHeight];
+			pTempBuffer4 = OGRE_ALLOC_T(uint, m_uWidth * m_uHeight, MEMCATEGORY_GENERAL);
 			for (y = 0; y < m_uHeight; y++)
 			{
 				dst4 = (pTempBuffer4 + ((y * m_uWidth) + m_uWidth - 1));
@@ -184,7 +184,7 @@ namespace Ogre {
 			}
 
 			memcpy(m_pBuffer, pTempBuffer4, m_uWidth * m_uHeight * sizeof(uint));
-			delete [] pTempBuffer4;
+			OGRE_FREE(pTempBuffer4, MEMCATEGORY_GENERAL);
 			break;
 
 		default:
@@ -214,7 +214,7 @@ namespace Ogre {
 
 		size_t rowSpan = m_uWidth * m_ucPixelSize;
 
-		uchar *pTempBuffer = new uchar[ rowSpan * m_uHeight ];
+		uchar *pTempBuffer = OGRE_ALLOC_T(uchar, rowSpan * m_uHeight, MEMCATEGORY_GENERAL);
 		uchar *ptr1 = m_pBuffer, *ptr2 = pTempBuffer + ( ( m_uHeight - 1 ) * rowSpan );
 
 		for( ushort i = 0; i < m_uHeight; i++ )
@@ -225,7 +225,7 @@ namespace Ogre {
 
 		memcpy( m_pBuffer, pTempBuffer, rowSpan * m_uHeight);
 
-		delete [] pTempBuffer;
+		OGRE_FREE(pTempBuffer, MEMCATEGORY_GENERAL);
 
 		return *this;
 	}
@@ -239,7 +239,7 @@ namespace Ogre {
 
 		if( m_pBuffer && m_bAutoDelete )
 		{
-			delete[] m_pBuffer;
+			OGRE_FREE(m_pBuffer, MEMCATEGORY_GENERAL);
 			m_pBuffer = NULL;
 		}
 		// Set image metadata
@@ -286,7 +286,7 @@ namespace Ogre {
 				"Image::loadRawData");
 		}
 
-		uchar *buffer = new uchar[ size ];
+		uchar *buffer = OGRE_ALLOC_T(uchar, size, MEMCATEGORY_GENERAL);
 		stream->read(buffer, size);
 
 		return loadDynamicImage(buffer,
@@ -294,60 +294,20 @@ namespace Ogre {
 			eFormat, true, numFaces, numMipMaps);
 
 	}
-
 	//-----------------------------------------------------------------------------
 	Image & Image::load(const String& strFileName, const String& group)
 	{
 
-		if( m_pBuffer && m_bAutoDelete )
-		{
-			delete[] m_pBuffer;
-			m_pBuffer = NULL;
-		}
-
 		String strExt;
 
 		size_t pos = strFileName.find_last_of(".");
-		if( pos == String::npos )
-			OGRE_EXCEPT(
-			Exception::ERR_INVALIDPARAMS, 
-			"Unable to load image file '" + strFileName + "' - invalid extension.",
-			"Image::load" );
+		if( pos != String::npos && pos < (strFileName.length() - 1))
+		{
+			strExt = strFileName.substr(pos+1);
+		}
 
-		while( pos != strFileName.length() - 1 )
-			strExt += strFileName[++pos];
-
-		Codec * pCodec = Codec::getCodec(strExt);
-		if( !pCodec )
-			OGRE_EXCEPT(
-			Exception::ERR_INVALIDPARAMS, 
-			"Unable to load image file '" + strFileName + "' - invalid extension.",
-			"Image::load" );
-
-		DataStreamPtr encoded = 
-			ResourceGroupManager::getSingleton().openResource(strFileName, group);
-
-		Codec::DecodeResult res = pCodec->decode(encoded);
-
-		ImageCodec::ImageData* pData = 
-			static_cast<ImageCodec::ImageData*>(res.second.getPointer());
-
-		// Get the format and compute the pixel size
-		m_uWidth = pData->width;
-		m_uHeight = pData->height;
-		m_uDepth = pData->depth;
-		m_uSize = pData->size;
-		m_eFormat = pData->format;
-		m_uNumMipmaps = pData->num_mipmaps;
-		m_ucPixelSize = static_cast<uchar>(PixelUtil::getNumElemBytes( m_eFormat ));
-		m_uFlags = pData->flags;
-
-		// re-use the decoded buffer
-		m_pBuffer = res.first->getPtr();
-		// ensure we don't delete when stream is closed
-		res.first->setFreeOnClose(false);
-
-		return *this;
+		DataStreamPtr encoded = ResourceGroupManager::getSingleton().openResource(strFileName, group);
+		return load(encoded, strExt);
 
 	}
 	//-----------------------------------------------------------------------------
@@ -377,7 +337,7 @@ namespace Ogre {
 			"Unable to save image file '" + filename + "' - invalid extension.",
 			"Image::save" );
 
-		ImageCodec::ImageData* imgData = new ImageCodec::ImageData();
+		ImageCodec::ImageData* imgData = OGRE_NEW ImageCodec::ImageData();
 		imgData->format = m_eFormat;
 		imgData->height = m_uHeight;
 		imgData->width = m_uWidth;
@@ -385,26 +345,70 @@ namespace Ogre {
 		// Wrap in CodecDataPtr, this will delete
 		Codec::CodecDataPtr codeDataPtr(imgData);
 		// Wrap memory, be sure not to delete when stream destroyed
-		MemoryDataStreamPtr wrapper(new MemoryDataStream(m_pBuffer, m_uSize, false));
+		MemoryDataStreamPtr wrapper(OGRE_NEW MemoryDataStream(m_pBuffer, m_uSize, false));
 
 		pCodec->codeToFile(wrapper, filename, codeDataPtr);
 	}
+	//---------------------------------------------------------------------
+	DataStreamPtr Image::encode(const String& formatextension)
+	{
+		if( !m_pBuffer )
+		{
+			OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "No image data loaded", 
+				"Image::encode");
+		}
+
+		Codec * pCodec = Codec::getCodec(formatextension);
+		if( !pCodec )
+			OGRE_EXCEPT(
+			Exception::ERR_INVALIDPARAMS, 
+			"Unable to encode image data as '" + formatextension + "' - invalid extension.",
+			"Image::encode" );
+
+		ImageCodec::ImageData* imgData = OGRE_NEW ImageCodec::ImageData();
+		imgData->format = m_eFormat;
+		imgData->height = m_uHeight;
+		imgData->width = m_uWidth;
+		imgData->depth = m_uDepth;
+		// Wrap in CodecDataPtr, this will delete
+		Codec::CodecDataPtr codeDataPtr(imgData);
+		// Wrap memory, be sure not to delete when stream destroyed
+		MemoryDataStreamPtr wrapper(OGRE_NEW MemoryDataStream(m_pBuffer, m_uSize, false));
+
+		return pCodec->code(wrapper, codeDataPtr);
+	}
 	//-----------------------------------------------------------------------------
 	Image & Image::load(DataStreamPtr& stream, const String& type )
 	{
 		if( m_pBuffer && m_bAutoDelete )
 		{
-			delete[] m_pBuffer;
+			OGRE_FREE(m_pBuffer, MEMCATEGORY_GENERAL);
 			m_pBuffer = NULL;
 		}
 
-		String strType = type;
+		Codec * pCodec = 0;
+		if (!type.empty())
+		{
+			// use named codec
+			pCodec = Codec::getCodec(type);
+		}
+		else
+		{
+			// derive from magic number
+			// read the first 32 bytes or file size, if less
+			size_t magicLen = std::min(stream->size(), (size_t)32);
+			char magicBuf[32];
+			stream->read(magicBuf, magicLen);
+			// return to start
+			stream->seek(0);
+			pCodec = Codec::getCodec(magicBuf, magicLen);
+		}
 
-		Codec * pCodec = Codec::getCodec(strType);
 		if( !pCodec )
 			OGRE_EXCEPT(
 			Exception::ERR_INVALIDPARAMS, 
-			"Unable to load image - invalid extension.",
+			"Unable to load image - unable to identify codec. Check file extension "
+			"and file format.",
 			"Image::load" );
 
 		Codec::DecodeResult res = pCodec->decode(stream);
@@ -429,7 +433,23 @@ namespace Ogre {
 
 		return *this;
 	}
+	//---------------------------------------------------------------------
+	String Image::getFileExtFromMagic(DataStreamPtr stream)
+	{
+		// read the first 32 bytes or file size, if less
+		size_t magicLen = std::min(stream->size(), (size_t)32);
+		char magicBuf[32];
+		stream->read(magicBuf, magicLen);
+		// return to start
+		stream->seek(0);
+		Codec* pCodec = Codec::getCodec(magicBuf, magicLen);
 
+		if(pCodec)
+			return pCodec->getType();
+		else
+			return StringUtil::BLANK;
+
+	}
 	//-----------------------------------------------------------------------------
 	uchar* Image::getData()
 	{
@@ -569,7 +589,7 @@ namespace Ogre {
 		m_uWidth = width;
 		m_uHeight = height;
 		m_uSize = PixelUtil::getMemorySize(m_uWidth, m_uHeight, 1, m_eFormat);
-		m_pBuffer = new uchar[m_uSize];
+		m_pBuffer = OGRE_ALLOC_T(uchar, m_uSize, MEMCATEGORY_GENERAL);
         m_uNumMipmaps = 0; // Loses precomputed mipmaps
 
 		// scale the image from temp into our resized buffer
@@ -595,7 +615,7 @@ namespace Ogre {
 			{
 				// Allocate temporary buffer of destination size in source format 
 				temp = PixelBox(scaled.getWidth(), scaled.getHeight(), scaled.getDepth(), src.format);
-				buf.bind(new MemoryDataStream(temp.getConsecutiveSize()));
+				buf.bind(OGRE_NEW MemoryDataStream(temp.getConsecutiveSize()));
 				temp.data = buf->getPtr();
 			}
 			// super-optimized: no conversion
@@ -638,7 +658,7 @@ namespace Ogre {
 				{
 					// Allocate temp buffer of destination size in source format 
 					temp = PixelBox(scaled.getWidth(), scaled.getHeight(), scaled.getDepth(), src.format);
-					buf.bind(new MemoryDataStream(temp.getConsecutiveSize()));
+					buf.bind(OGRE_NEW MemoryDataStream(temp.getConsecutiveSize()));
 					temp.data = buf->getPtr();
 				}
 				// super-optimized: byte-oriented math, no conversion
@@ -677,7 +697,7 @@ namespace Ogre {
 
 	//-----------------------------------------------------------------------------    
 
-	ColourValue Image::getColourAt(int x, int y, int z) 
+	ColourValue Image::getColourAt(int x, int y, int z) const
 	{
 		ColourValue rval;
 		PixelUtil::unpackColour(&rval, m_eFormat, &m_pBuffer[m_ucPixelSize * (z * m_uWidth * m_uHeight + m_uWidth * y + x)]);
