@@ -50,8 +50,15 @@ namespace Ogre {
 		  mAttenuationLinear(0.0f),
           mAttenuationQuad(0.0f),
 		  mPowerScale(1.0f),
+		  mIndexInFrame(0),
+		  mOwnShadowFarDist(false),
+		  mShadowFarDist(0),
+		  mShadowFarDistSquared(0),
           mDerivedPosition(Vector3::ZERO),
           mDerivedDirection(Vector3::UNIT_Z),
+		  mDerivedCamRelativePosition(Vector3::ZERO),
+		  mDerivedCamRelativeDirty(false),
+		  mCameraToBeRelativeTo(0),
           mDerivedTransformDirty(false),
 		  mCustomShadowCameraSetup()
     {
@@ -71,8 +78,14 @@ namespace Ogre {
 		mAttenuationLinear(0.0f),
         mAttenuationQuad(0.0f),
 		mPowerScale(1.0f),
+		mIndexInFrame(0),
+		mOwnShadowFarDist(false),
+		mShadowFarDist(0),
+		mShadowFarDistSquared(0),
         mDerivedPosition(Vector3::ZERO),
         mDerivedDirection(Vector3::UNIT_Z),
+		mDerivedCamRelativeDirty(false),
+		mCameraToBeRelativeTo(0),
         mDerivedTransformDirty(false),
 		mCustomShadowCameraSetup()
     {
@@ -98,6 +111,7 @@ namespace Ogre {
         mPosition.y = y;
         mPosition.z = z;
         mDerivedTransformDirty = true;
+		mDerivedCamRelativeDirty = true;
 
     }
     //-----------------------------------------------------------------------
@@ -105,6 +119,7 @@ namespace Ogre {
     {
         mPosition = vec;
         mDerivedTransformDirty = true;
+		mDerivedCamRelativeDirty = true;
     }
     //-----------------------------------------------------------------------
     const Vector3& Light::getPosition(void) const
@@ -267,6 +282,11 @@ namespace Ogre {
 
             mDerivedTransformDirty = false;
         }
+		if (mCameraToBeRelativeTo && mDerivedCamRelativeDirty)
+		{
+			mDerivedCamRelativePosition = mDerivedPosition - mCameraToBeRelativeTo->getDerivedPosition();
+			mDerivedCamRelativeDirty = false;
+		}
     }
     //-----------------------------------------------------------------------
     void Light::_notifyAttached(Node* parent, bool isTagPoint)
@@ -294,16 +314,29 @@ namespace Ogre {
     {
         // Do nothing
     }
+	//-----------------------------------------------------------------------
+	void Light::visitRenderables(Renderable::Visitor* visitor, 
+		bool debugRenderables)
+	{
+		// nothing to render
+	}
     //-----------------------------------------------------------------------
     const String& Light::getMovableType(void) const
     {
 		return LightFactory::FACTORY_TYPE_NAME;
     }
     //-----------------------------------------------------------------------
-    const Vector3& Light::getDerivedPosition(void) const
+    const Vector3& Light::getDerivedPosition(bool cameraRelative) const
     {
         update();
-        return mDerivedPosition;
+		if (cameraRelative && mCameraToBeRelativeTo)
+		{
+			return mDerivedCamRelativePosition;
+		}
+		else
+		{
+			return mDerivedPosition;
+		}
     }
     //-----------------------------------------------------------------------
     const Vector3& Light::getDerivedDirection(void) const
@@ -317,7 +350,7 @@ namespace Ogre {
         MovableObject::setVisible(visible);
     }
     //-----------------------------------------------------------------------
-	Vector4 Light::getAs4DVector(void) const
+	Vector4 Light::getAs4DVector(bool cameraRelativeIfSet) const
 	{
 		Vector4 ret;
         if (mLightType == Light::LT_DIRECTIONAL)
@@ -327,7 +360,7 @@ namespace Ogre {
         }	
 		else
         {
-            ret = getDerivedPosition();
+            ret = getDerivedPosition(cameraRelativeIfSet);
             ret.w = 1.0;
         }
 		return ret;
@@ -684,32 +717,32 @@ namespace Ogre {
 		if (valueName == "diffuseColour")
 		{
 			return AnimableValuePtr(
-				new LightDiffuseColourValue(this));
+				OGRE_NEW LightDiffuseColourValue(this));
 		}
 		else if(valueName == "specularColour")
 		{
 			return AnimableValuePtr(
-				new LightSpecularColourValue(this));
+				OGRE_NEW LightSpecularColourValue(this));
 		}
 		else if (valueName == "attenuation")
 		{
 			return AnimableValuePtr(
-				new LightAttenuationValue(this));
+				OGRE_NEW LightAttenuationValue(this));
 		}
 		else if (valueName == "spotlightInner")
 		{
 			return AnimableValuePtr(
-				new LightSpotlightInnerValue(this));
+				OGRE_NEW LightSpotlightInnerValue(this));
 		}
 		else if (valueName == "spotlightOuter")
 		{
 			return AnimableValuePtr(
-				new LightSpotlightOuterValue(this));
+				OGRE_NEW LightSpotlightOuterValue(this));
 		}
 		else if (valueName == "spotlightFalloff")
 		{
 			return AnimableValuePtr(
-				new LightSpotlightFalloffValue(this));
+				OGRE_NEW LightSpotlightFalloffValue(this));
 		}
 		else
 		{
@@ -732,6 +765,40 @@ namespace Ogre {
 		return mCustomShadowCameraSetup;
 	}
 	//-----------------------------------------------------------------------
+	void Light::setShadowFarDistance(Real distance)
+	{
+		mOwnShadowFarDist = true;
+		mShadowFarDist = distance;
+		mShadowFarDistSquared = distance * distance;
+	}
+	//-----------------------------------------------------------------------
+	void Light::resetShadowFarDistance(void)
+	{
+		mOwnShadowFarDist = false;
+	}
+	//-----------------------------------------------------------------------
+	Real Light::getShadowFarDistance(void) const
+	{
+		if (mOwnShadowFarDist)
+			return mShadowFarDist;
+		else
+			return mManager->getShadowFarDistance ();
+	}
+	//-----------------------------------------------------------------------
+	Real Light::getShadowFarDistanceSquared(void) const
+	{
+		if (mOwnShadowFarDist)
+			return mShadowFarDistSquared;
+		else
+			return mManager->getShadowFarDistanceSquared ();
+	}
+	//---------------------------------------------------------------------
+	void Light::_setCameraRelative(Camera* cam)
+	{
+		mCameraToBeRelativeTo = cam;
+		mDerivedCamRelativeDirty = true;
+	}
+	//-----------------------------------------------------------------------
 	//-----------------------------------------------------------------------
 	String LightFactory::FACTORY_TYPE_NAME = "Light";
 	//-----------------------------------------------------------------------
@@ -744,13 +811,76 @@ namespace Ogre {
 		const NameValuePairList* params)
 	{
 
-		return new Light(name);
+		Light* light = OGRE_NEW Light(name);
+ 
+		if(params)
+		{
+			NameValuePairList::const_iterator ni;
 
+			// Setting the light type first before any property specific to a certain light type
+			if ((ni = params->find("type")) != params->end())
+			{
+				if (ni->second == "point")
+					light->setType(Light::LT_POINT);
+				else if (ni->second == "directional")
+					light->setType(Light::LT_DIRECTIONAL);
+				else if (ni->second == "spotlight")
+					light->setType(Light::LT_SPOTLIGHT);
+				else
+					OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS,
+						"Invalid light type '" + ni->second + "'.",
+						"LightFactory::createInstance");
+			}
+
+			// Common properties
+			if ((ni = params->find("position")) != params->end())
+				light->setPosition(StringConverter::parseVector3(ni->second));
+
+			if ((ni = params->find("direction")) != params->end())
+				light->setDirection(StringConverter::parseVector3(ni->second));
+
+			if ((ni = params->find("diffuseColour")) != params->end())
+				light->setDiffuseColour(StringConverter::parseColourValue(ni->second));
+
+			if ((ni = params->find("specularColour")) != params->end())
+				light->setSpecularColour(StringConverter::parseColourValue(ni->second));
+
+			if ((ni = params->find("attenuation")) != params->end())
+			{
+				Vector4 attenuation = StringConverter::parseVector4(ni->second);
+				light->setAttenuation(attenuation.x, attenuation.y, attenuation.z, attenuation.w);
+			}
+
+			if ((ni = params->find("castShadows")) != params->end())
+				light->setCastShadows(StringConverter::parseBool(ni->second));
+
+			if ((ni = params->find("visible")) != params->end())
+				light->setVisible(StringConverter::parseBool(ni->second));
+
+			if ((ni = params->find("powerScale")) != params->end())
+				light->setPowerScale(StringConverter::parseReal(ni->second));
+
+			if ((ni = params->find("shadowFarDistance")) != params->end())
+				light->setShadowFarDistance(StringConverter::parseReal(ni->second));
+
+
+			// Spotlight properties
+			if ((ni = params->find("spotlightInner")) != params->end())
+				light->setSpotlightInnerAngle(StringConverter::parseAngle(ni->second));
+
+			if ((ni = params->find("spotlightOuter")) != params->end())
+				light->setSpotlightOuterAngle(StringConverter::parseAngle(ni->second));
+
+			if ((ni = params->find("spotlightFalloff")) != params->end())
+				light->setSpotlightFalloff(StringConverter::parseReal(ni->second));
+		}
+
+		return light;
 	}
 	//-----------------------------------------------------------------------
 	void LightFactory::destroyInstance( MovableObject* obj)
 	{
-		delete obj;
+		OGRE_DELETE obj;
 	}
 
 
