@@ -45,6 +45,7 @@ namespace Ogre {
 	{
 		uint32 operator()(const Pass* p) const
 		{
+			OGRE_LOCK_MUTEX(p->mTexUnitChangeMutex)
 
 			_StringHash H;
 			uint32 hash = p->getIndex() << 28;
@@ -76,6 +77,7 @@ namespace Ogre {
 	{
 		uint32 operator()(const Pass* p) const
 		{
+			OGRE_LOCK_MUTEX(p->mGpuProgramChangeMutex)
 
 			_StringHash H;
 			uint32 hash = p->getIndex() << 28;
@@ -123,14 +125,20 @@ namespace Ogre {
 		, mTracking(TVC_NONE)
 		, mSourceBlendFactor(SBF_ONE)
 		, mDestBlendFactor(SBF_ZERO)
+		, mSourceBlendFactorAlpha(SBF_ONE)
+		, mDestBlendFactorAlpha(SBF_ZERO)
+		, mSeparateBlend(false)
 		, mDepthCheck(true)
 		, mDepthWrite(true)
 		, mDepthFunc(CMPF_LESS_EQUAL)
 		, mDepthBiasConstant(0.0f)
 		, mDepthBiasSlopeScale(0.0f)
+		, mDepthBiasPerIteration(0.0f)
 		, mColourWrite(true)
 		, mAlphaRejectFunc(CMPF_ALWAYS_PASS)
 		, mAlphaRejectVal(0)
+		, mAlphaToCoverageEnabled(false)
+		, mTransparentSorting(true)
 		, mCullMode(CULL_CLOCKWISE)
 		, mManualCullMode(MANUAL_CULL_BACK)
 		, mLightingEnabled(true)
@@ -142,6 +150,8 @@ namespace Ogre {
 		, mOnlyLightType(Light::LT_POINT)
 		, mShadeOptions(SO_GOURAUD)
 		, mPolygonMode(PM_SOLID)
+		, mNormaliseNormals(false)
+		, mPolygonModeOverrideable(true)
 		, mFogOverride(false)
 		, mFogMode(FOG_NONE)
 		, mFogColour(ColourValue::White)
@@ -152,6 +162,7 @@ namespace Ogre {
 		, mShadowCasterVertexProgramUsage(0)
 		, mShadowReceiverVertexProgramUsage(0)
 		, mFragmentProgramUsage(0)
+		, mGeometryProgramUsage(0)
 		, mShadowReceiverFragmentProgramUsage(0)
 		, mQueuedForDeletion(false)
 		, mPassIterationCount(1)
@@ -161,6 +172,9 @@ namespace Ogre {
 		, mPointSpritesEnabled(false)
 		, mPointAttenuationEnabled(false)
 		, mContentTypeLookupBuilt(false)
+		, mLightScissoring(false)
+		, mLightClipPlanes(false)
+		, mIlluminationStage(IS_UNKNOWN)
     {
 		mPointAttenuationCoeffs[0] = 1.0f;
 		mPointAttenuationCoeffs[1] = mPointAttenuationCoeffs[2] = 0.0f;
@@ -209,15 +223,21 @@ namespace Ogre {
 	    // Default blending (overwrite)
 	    mSourceBlendFactor = oth.mSourceBlendFactor;
 	    mDestBlendFactor = oth.mDestBlendFactor;
+		mSourceBlendFactorAlpha = oth.mSourceBlendFactorAlpha;
+		mDestBlendFactorAlpha = oth.mDestBlendFactorAlpha;
+		mSeparateBlend = oth.mSeparateBlend;
 
 	    mDepthCheck = oth.mDepthCheck;
 	    mDepthWrite = oth.mDepthWrite;
 		mAlphaRejectFunc = oth.mAlphaRejectFunc;
 		mAlphaRejectVal = oth.mAlphaRejectVal;
+		mAlphaToCoverageEnabled = oth.mAlphaToCoverageEnabled;
+		mTransparentSorting = oth.mTransparentSorting;
         mColourWrite = oth.mColourWrite;
 	    mDepthFunc = oth.mDepthFunc;
         mDepthBiasConstant = oth.mDepthBiasConstant;
 		mDepthBiasSlopeScale = oth.mDepthBiasSlopeScale;
+		mDepthBiasPerIteration = oth.mDepthBiasPerIteration;
 	    mCullMode = oth.mCullMode;
 	    mManualCullMode = oth.mManualCullMode;
 	    mLightingEnabled = oth.mLightingEnabled;
@@ -226,9 +246,11 @@ namespace Ogre {
 		mIteratePerLight = oth.mIteratePerLight;
 		mLightsPerIteration = oth.mLightsPerIteration;
         mRunOnlyForOneLightType = oth.mRunOnlyForOneLightType;
+		mNormaliseNormals = oth.mNormaliseNormals;
         mOnlyLightType = oth.mOnlyLightType;
 	    mShadeOptions = oth.mShadeOptions;
 		mPolygonMode = oth.mPolygonMode;
+		mPolygonModeOverrideable = oth.mPolygonModeOverrideable;
         mPassIterationCount = oth.mPassIterationCount;
 		mPointSize = oth.mPointSize;
 		mPointMinSize = oth.mPointMinSize;
@@ -238,11 +260,14 @@ namespace Ogre {
 		memcpy(mPointAttenuationCoeffs, oth.mPointAttenuationCoeffs, sizeof(Real)*3);
 		mShadowContentTypeLookup = oth.mShadowContentTypeLookup;
 		mContentTypeLookupBuilt = oth.mContentTypeLookupBuilt;
+		mLightScissoring = oth.mLightScissoring;
+		mLightClipPlanes = oth.mLightClipPlanes;
+		mIlluminationStage = oth.mIlluminationStage;
 
 
 		if (oth.mVertexProgramUsage)
 		{
-			mVertexProgramUsage = new GpuProgramUsage(*(oth.mVertexProgramUsage));
+			mVertexProgramUsage = OGRE_NEW GpuProgramUsage(*(oth.mVertexProgramUsage));
 		}
 		else
 		{
@@ -250,7 +275,7 @@ namespace Ogre {
 		}
         if (oth.mShadowCasterVertexProgramUsage)
         {
-            mShadowCasterVertexProgramUsage = new GpuProgramUsage(*(oth.mShadowCasterVertexProgramUsage));
+            mShadowCasterVertexProgramUsage = OGRE_NEW GpuProgramUsage(*(oth.mShadowCasterVertexProgramUsage));
         }
         else
         {
@@ -258,7 +283,7 @@ namespace Ogre {
         }
         if (oth.mShadowReceiverVertexProgramUsage)
         {
-            mShadowReceiverVertexProgramUsage = new GpuProgramUsage(*(oth.mShadowReceiverVertexProgramUsage));
+            mShadowReceiverVertexProgramUsage = OGRE_NEW GpuProgramUsage(*(oth.mShadowReceiverVertexProgramUsage));
         }
         else
         {
@@ -266,15 +291,23 @@ namespace Ogre {
         }
 		if (oth.mFragmentProgramUsage)
 		{
-		    mFragmentProgramUsage = new GpuProgramUsage(*(oth.mFragmentProgramUsage));
+		    mFragmentProgramUsage = OGRE_NEW GpuProgramUsage(*(oth.mFragmentProgramUsage));
         }
         else
         {
 		    mFragmentProgramUsage = NULL;
+        }
+		if (oth.mGeometryProgramUsage)
+		{
+		    mGeometryProgramUsage = OGRE_NEW GpuProgramUsage(*(oth.mGeometryProgramUsage));
+        }
+        else
+        {
+		    mGeometryProgramUsage = NULL;
         }
 		if (oth.mShadowReceiverFragmentProgramUsage)
 		{
-			mShadowReceiverFragmentProgramUsage = new GpuProgramUsage(*(oth.mShadowReceiverFragmentProgramUsage));
+			mShadowReceiverFragmentProgramUsage = OGRE_NEW GpuProgramUsage(*(oth.mShadowReceiverFragmentProgramUsage));
 		}
 		else
 		{
@@ -288,7 +321,7 @@ namespace Ogre {
         iend = mTextureUnitStates.end();
         for (i = mTextureUnitStates.begin(); i != iend; ++i)
         {
-            delete *i;
+            OGRE_DELETE *i;
         }
 
         mTextureUnitStates.clear();
@@ -297,7 +330,7 @@ namespace Ogre {
 		iend = oth.mTextureUnitStates.end();
 		for (i = oth.mTextureUnitStates.begin(); i != iend; ++i)
 		{
-			TextureUnitState* t = new TextureUnitState(this, *(*i));
+			TextureUnitState* t = OGRE_NEW TextureUnitState(this, *(*i));
 			mTextureUnitStates.push_back(t);
 		}
 
@@ -474,7 +507,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     TextureUnitState* Pass::createTextureUnitState(void)
     {
-        TextureUnitState *t = new TextureUnitState(this);
+        TextureUnitState *t = OGRE_NEW TextureUnitState(this);
         addTextureUnitState(t);
 		mContentTypeLookupBuilt = false;
 	    return t;
@@ -483,7 +516,7 @@ namespace Ogre {
     TextureUnitState* Pass::createTextureUnitState(
         const String& textureName, unsigned short texCoordSet)
     {
-        TextureUnitState *t = new TextureUnitState(this);
+        TextureUnitState *t = OGRE_NEW TextureUnitState(this);
 	    t->setTextureName(textureName);
 	    t->setTextureCoordSet(texCoordSet);
         addTextureUnitState(t);
@@ -493,6 +526,8 @@ namespace Ogre {
     //-----------------------------------------------------------------------
 	void Pass::addTextureUnitState(TextureUnitState* state)
 	{
+		OGRE_LOCK_MUTEX(mTexUnitChangeMutex)
+
         assert(state && "state is 0 in Pass::addTextureUnitState()");
         if (state)
         {
@@ -530,12 +565,14 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     TextureUnitState* Pass::getTextureUnitState(unsigned short index)
     {
+		OGRE_LOCK_MUTEX(mTexUnitChangeMutex)
         assert (index < mTextureUnitStates.size() && "Index out of bounds");
 	    return mTextureUnitStates[index];
     }
     //-----------------------------------------------------------------------------
     TextureUnitState* Pass::getTextureUnitState(const String& name)
     {
+		OGRE_LOCK_MUTEX(mTexUnitChangeMutex)
         TextureUnitStates::iterator i    = mTextureUnitStates.begin();
         TextureUnitStates::iterator iend = mTextureUnitStates.end();
         TextureUnitState* foundTUS = 0;
@@ -557,12 +594,14 @@ namespace Ogre {
 	//-----------------------------------------------------------------------
 	const TextureUnitState* Pass::getTextureUnitState(unsigned short index) const
 	{
+		OGRE_LOCK_MUTEX(mTexUnitChangeMutex)
 		assert (index < mTextureUnitStates.size() && "Index out of bounds");
 		return mTextureUnitStates[index];
 	}
 	//-----------------------------------------------------------------------------
 	const TextureUnitState* Pass::getTextureUnitState(const String& name) const
 	{
+		OGRE_LOCK_MUTEX(mTexUnitChangeMutex)
 		TextureUnitStates::const_iterator i    = mTextureUnitStates.begin();
 		TextureUnitStates::const_iterator iend = mTextureUnitStates.end();
 		const TextureUnitState* foundTUS = 0;
@@ -585,6 +624,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     unsigned short Pass::getTextureUnitStateIndex(const TextureUnitState* state) const
     {
+		OGRE_LOCK_MUTEX(mTexUnitChangeMutex)
         assert(state && "state is 0 in Pass::getTextureUnitStateIndex()");
 
         // only find index for state attached to this pass
@@ -617,10 +657,11 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void Pass::removeTextureUnitState(unsigned short index)
     {
+		OGRE_LOCK_MUTEX(mTexUnitChangeMutex)
         assert (index < mTextureUnitStates.size() && "Index out of bounds");
 
         TextureUnitStates::iterator i = mTextureUnitStates.begin() + index;
-        delete *i;
+        OGRE_DELETE *i;
 	    mTextureUnitStates.erase(i);
         if (!mQueuedForDeletion)
         {
@@ -633,11 +674,12 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void Pass::removeAllTextureUnitStates(void)
     {
+		OGRE_LOCK_MUTEX(mTexUnitChangeMutex)
         TextureUnitStates::iterator i, iend;
         iend = mTextureUnitStates.end();
         for (i = mTextureUnitStates.begin(); i != iend; ++i)
         {
-            delete *i;
+            OGRE_DELETE *i;
         }
         mTextureUnitStates.clear();
         if (!mQueuedForDeletion)
@@ -648,37 +690,87 @@ namespace Ogre {
         _dirtyHash();
 		mContentTypeLookupBuilt = false;
     }
+	//-----------------------------------------------------------------------
+	void Pass::_getBlendFlags(SceneBlendType type, SceneBlendFactor& source, SceneBlendFactor& dest)
+	{
+		switch ( type )
+		{
+	    case SBT_TRANSPARENT_ALPHA:
+		    source = SBF_SOURCE_ALPHA;
+			dest = SBF_ONE_MINUS_SOURCE_ALPHA;
+		    return;
+	    case SBT_TRANSPARENT_COLOUR:
+		    source = SBF_SOURCE_COLOUR;
+			dest = SBF_ONE_MINUS_SOURCE_COLOUR;
+		    return;
+		case SBT_MODULATE:
+			source = SBF_DEST_COLOUR;
+			dest = SBF_ZERO;
+			return;
+	    case SBT_ADD:
+		    source = SBF_ONE;
+			dest = SBF_ONE;
+		    return;
+        case SBT_REPLACE:
+            source = SBF_ONE;
+			dest = SBF_ZERO;
+            return;
+		}
+
+		// Default to SBT_REPLACE
+
+		source = SBF_ONE;
+		dest = SBF_ZERO;
+	}
     //-----------------------------------------------------------------------
     void Pass::setSceneBlending(SceneBlendType sbt)
     {
-	    // Turn predefined type into blending factors
-	    switch (sbt)
-	    {
-	    case SBT_TRANSPARENT_ALPHA:
-		    setSceneBlending(SBF_SOURCE_ALPHA, SBF_ONE_MINUS_SOURCE_ALPHA);
-		    break;
-	    case SBT_TRANSPARENT_COLOUR:
-		    setSceneBlending(SBF_SOURCE_COLOUR, SBF_ONE_MINUS_SOURCE_COLOUR);
-		    break;
-		case SBT_MODULATE:
-			setSceneBlending(SBF_DEST_COLOUR, SBF_ZERO);
-			break;
-	    case SBT_ADD:
-		    setSceneBlending(SBF_ONE, SBF_ONE);
-		    break;
-        case SBT_REPLACE:
-            setSceneBlending(SBF_ONE, SBF_ZERO);
-            break;
-	    // TODO: more
-	    }
+		// Convert type into blend factors
 
+		SceneBlendFactor source;
+		SceneBlendFactor dest;
+		_getBlendFlags(sbt, source, dest);
+
+		// Set blend factors
+
+		setSceneBlending(source, dest);
     }
+	//-----------------------------------------------------------------------
+	void Pass::setSeparateSceneBlending( const SceneBlendType sbt, const SceneBlendType sbta )
+	{
+		// Convert types into blend factors
+
+		SceneBlendFactor source;
+		SceneBlendFactor dest;
+		_getBlendFlags(sbt, source, dest);
+
+		SceneBlendFactor sourceAlpha;
+		SceneBlendFactor destAlpha;
+		_getBlendFlags(sbta, sourceAlpha, destAlpha);
+
+		// Set blend factors
+
+		setSeparateSceneBlending(source, dest, sourceAlpha, destAlpha);
+	}
+
     //-----------------------------------------------------------------------
     void Pass::setSceneBlending(SceneBlendFactor sourceFactor, SceneBlendFactor destFactor)
     {
 	    mSourceBlendFactor = sourceFactor;
 	    mDestBlendFactor = destFactor;
+
+		mSeparateBlend = false;
     }
+	//-----------------------------------------------------------------------
+	void Pass::setSeparateSceneBlending( const SceneBlendFactor sourceFactor, const SceneBlendFactor destFactor, const SceneBlendFactor sourceFactorAlpha, const SceneBlendFactor destFactorAlpha )
+	{
+		mSourceBlendFactor = sourceFactor;
+		mDestBlendFactor = destFactor;
+		mSourceBlendFactorAlpha = sourceFactorAlpha;
+		mDestBlendFactorAlpha = destFactorAlpha;
+
+		mSeparateBlend = true;
+	}
     //-----------------------------------------------------------------------
     SceneBlendFactor Pass::getSourceBlendFactor(void) const
     {
@@ -690,6 +782,21 @@ namespace Ogre {
 	    return mDestBlendFactor;
     }
     //-----------------------------------------------------------------------
+    SceneBlendFactor Pass::getSourceBlendFactorAlpha(void) const
+    {
+	    return mSourceBlendFactorAlpha;
+    }
+    //-----------------------------------------------------------------------
+    SceneBlendFactor Pass::getDestBlendFactorAlpha(void) const
+    {
+	    return mDestBlendFactorAlpha;
+    }
+	//-----------------------------------------------------------------------
+	bool Pass::hasSeparateSceneBlending() const
+	{
+		return mSeparateBlend;
+	}
+    //-----------------------------------------------------------------------
     bool Pass::isTransparent(void) const
     {
 		// Transparent if any of the destination colour is taken into account
@@ -737,10 +844,11 @@ namespace Ogre {
 	    return mDepthFunc;
     }
 	//-----------------------------------------------------------------------
-	void Pass::setAlphaRejectSettings(CompareFunction func, unsigned char value)
+	void Pass::setAlphaRejectSettings(CompareFunction func, unsigned char value, bool alphaToCoverage)
 	{
 		mAlphaRejectFunc = func;
 		mAlphaRejectVal = value;
+		mAlphaToCoverageEnabled = alphaToCoverage;
 	}
 	//-----------------------------------------------------------------------
 	void Pass::setAlphaRejectFunction(CompareFunction func)
@@ -752,6 +860,21 @@ namespace Ogre {
 	{
 		mAlphaRejectVal = val;
 	}
+	//---------------------------------------------------------------------
+	void Pass::setAlphaToCoverageEnabled(bool enabled)
+	{
+		mAlphaToCoverageEnabled = enabled;
+	}
+	//-----------------------------------------------------------------------
+	void Pass::setTransparentSortingEnabled(bool enabled)
+	{
+		mTransparentSorting = enabled;
+	}
+	//-----------------------------------------------------------------------
+	bool Pass::getTransparentSortingEnabled(void) const
+	{
+		return mTransparentSorting;
+	}
     //-----------------------------------------------------------------------
 	void Pass::setColourWriteEnabled(bool enabled)
 	{
@@ -909,10 +1032,20 @@ namespace Ogre {
 	{
 		return mDepthBiasSlopeScale;
 	}
+	//---------------------------------------------------------------------
+	void Pass::setIterationDepthBias(float biasPerIteration)
+	{
+		mDepthBiasPerIteration = biasPerIteration;
+	}
+	//---------------------------------------------------------------------
+	float Pass::getIterationDepthBias() const
+	{
+		return mDepthBiasPerIteration;
+	}
     //-----------------------------------------------------------------------
 	Pass* Pass::_split(unsigned short numUnits)
 	{
-		if (mVertexProgramUsage || mFragmentProgramUsage)
+		if (mVertexProgramUsage || mGeometryProgramUsage || mFragmentProgramUsage)
 		{
 			OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Programmable passes cannot be "
 				"automatically split, define a fallback technique instead.",
@@ -962,6 +1095,33 @@ namespace Ogre {
 			_dirtyHash();
 		}
 	}
+    //-----------------------------------------------------------------------
+	void Pass::_prepare(void)
+	{
+		// We assume the Technique only calls this when the material is being
+		// prepared
+
+		// prepare each TextureUnitState
+		TextureUnitStates::iterator i, iend;
+		iend = mTextureUnitStates.end();
+		for (i = mTextureUnitStates.begin(); i != iend; ++i)
+		{
+			(*i)->_prepare();
+		}
+
+	}
+    //-----------------------------------------------------------------------
+	void Pass::_unprepare(void)
+	{
+		// unprepare each TextureUnitState
+		TextureUnitStates::iterator i, iend;
+		iend = mTextureUnitStates.end();
+		for (i = mTextureUnitStates.begin(); i != iend; ++i)
+		{
+			(*i)->_unprepare();
+		}
+
+	}
     //-----------------------------------------------------------------------
 	void Pass::_load(void)
 	{
@@ -993,6 +1153,12 @@ namespace Ogre {
             mShadowReceiverVertexProgramUsage->_load();
         }
 
+		if (mGeometryProgramUsage)
+		{
+			// Load vertex program
+            mGeometryProgramUsage->_load();
+        }
+
         if (mFragmentProgramUsage)
         {
 			// Load fragment program
@@ -1021,6 +1187,10 @@ namespace Ogre {
 		{
 			// TODO
 		}
+		if (mGeometryProgramUsage)
+		{
+			// TODO
+		}
         if (mFragmentProgramUsage)
         {
             // TODO
@@ -1029,17 +1199,19 @@ namespace Ogre {
     //-----------------------------------------------------------------------
 	void Pass::setVertexProgram(const String& name, bool resetParams)
 	{
+		OGRE_LOCK_MUTEX(mGpuProgramChangeMutex)
+
         // Turn off vertex program if name blank
         if (name.empty())
         {
-            if (mVertexProgramUsage) delete mVertexProgramUsage;
+            if (mVertexProgramUsage) OGRE_DELETE mVertexProgramUsage;
             mVertexProgramUsage = NULL;
         }
         else
         {
             if (!mVertexProgramUsage)
             {
-                mVertexProgramUsage = new GpuProgramUsage(GPT_VERTEX_PROGRAM);
+                mVertexProgramUsage = OGRE_NEW GpuProgramUsage(GPT_VERTEX_PROGRAM);
             }
 		    mVertexProgramUsage->setProgramName(name, resetParams);
         }
@@ -1049,6 +1221,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------
 	void Pass::setVertexProgramParameters(GpuProgramParametersSharedPtr params)
 	{
+		OGRE_LOCK_MUTEX(mGpuProgramChangeMutex)
 		if (!mVertexProgramUsage)
         {
             OGRE_EXCEPT (Exception::ERR_INVALIDPARAMS,
@@ -1060,17 +1233,18 @@ namespace Ogre {
     //-----------------------------------------------------------------------
 	void Pass::setFragmentProgram(const String& name, bool resetParams)
 	{
+		OGRE_LOCK_MUTEX(mGpuProgramChangeMutex)
         // Turn off fragment program if name blank
         if (name.empty())
         {
-            if (mFragmentProgramUsage) delete mFragmentProgramUsage;
+            if (mFragmentProgramUsage) OGRE_DELETE mFragmentProgramUsage;
             mFragmentProgramUsage = NULL;
         }
         else
         {
             if (!mFragmentProgramUsage)
             {
-                mFragmentProgramUsage = new GpuProgramUsage(GPT_FRAGMENT_PROGRAM);
+                mFragmentProgramUsage = OGRE_NEW GpuProgramUsage(GPT_FRAGMENT_PROGRAM);
             }
 		    mFragmentProgramUsage->setProgramName(name, resetParams);
         }
@@ -1080,6 +1254,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------
 	void Pass::setFragmentProgramParameters(GpuProgramParametersSharedPtr params)
 	{
+		OGRE_LOCK_MUTEX(mGpuProgramChangeMutex)
 		if (!mFragmentProgramUsage)
         {
             OGRE_EXCEPT (Exception::ERR_INVALIDPARAMS,
@@ -1089,8 +1264,42 @@ namespace Ogre {
 		mFragmentProgramUsage->setParameters(params);
 	}
 	//-----------------------------------------------------------------------
+	void Pass::setGeometryProgram(const String& name, bool resetParams)
+	{
+		OGRE_LOCK_MUTEX(mGpuProgramChangeMutex)
+        // Turn off geometry program if name blank
+        if (name.empty())
+        {
+            if (mGeometryProgramUsage) OGRE_DELETE mGeometryProgramUsage;
+            mGeometryProgramUsage = NULL;
+        }
+        else
+        {
+            if (!mGeometryProgramUsage)
+            {
+                mGeometryProgramUsage = OGRE_NEW GpuProgramUsage(GPT_GEOMETRY_PROGRAM);
+            }
+		    mGeometryProgramUsage->setProgramName(name, resetParams);
+        }
+        // Needs recompilation
+        mParent->_notifyNeedsRecompile();
+	}
+    //-----------------------------------------------------------------------
+	void Pass::setGeometryProgramParameters(GpuProgramParametersSharedPtr params)
+	{
+		OGRE_LOCK_MUTEX(mGpuProgramChangeMutex)
+		if (!mGeometryProgramUsage)
+        {
+            OGRE_EXCEPT (Exception::ERR_INVALIDPARAMS,
+                "This pass does not have a geometry program assigned!",
+                "Pass::setGeometryProgramParameters");
+        }
+		mGeometryProgramUsage->setParameters(params);
+	}
+	//-----------------------------------------------------------------------
 	const String& Pass::getVertexProgramName(void) const
 	{
+		OGRE_LOCK_MUTEX(mGpuProgramChangeMutex)
         if (!mVertexProgramUsage)
             return StringUtil::BLANK;
         else
@@ -1099,6 +1308,7 @@ namespace Ogre {
 	//-----------------------------------------------------------------------
 	GpuProgramParametersSharedPtr Pass::getVertexProgramParameters(void) const
 	{
+		OGRE_LOCK_MUTEX(mGpuProgramChangeMutex)
 		if (!mVertexProgramUsage)
         {
             OGRE_EXCEPT (Exception::ERR_INVALIDPARAMS,
@@ -1110,11 +1320,13 @@ namespace Ogre {
 	//-----------------------------------------------------------------------
 	const GpuProgramPtr& Pass::getVertexProgram(void) const
 	{
+		OGRE_LOCK_MUTEX(mGpuProgramChangeMutex)
 		return mVertexProgramUsage->getProgram();
 	}
 	//-----------------------------------------------------------------------
 	const String& Pass::getFragmentProgramName(void) const
 	{
+		OGRE_LOCK_MUTEX(mGpuProgramChangeMutex)
         if (!mFragmentProgramUsage)
             return StringUtil::BLANK;
         else
@@ -1123,14 +1335,37 @@ namespace Ogre {
 	//-----------------------------------------------------------------------
 	GpuProgramParametersSharedPtr Pass::getFragmentProgramParameters(void) const
 	{
+		OGRE_LOCK_MUTEX(mGpuProgramChangeMutex)
 		return mFragmentProgramUsage->getParameters();
 	}
 	//-----------------------------------------------------------------------
 	const GpuProgramPtr& Pass::getFragmentProgram(void) const
 	{
+		OGRE_LOCK_MUTEX(mGpuProgramChangeMutex)
 		return mFragmentProgramUsage->getProgram();
 	}
 	//-----------------------------------------------------------------------
+	const String& Pass::getGeometryProgramName(void) const
+	{
+		OGRE_LOCK_MUTEX(mGpuProgramChangeMutex)
+        if (!mGeometryProgramUsage)
+            return StringUtil::BLANK;
+        else
+    		return mGeometryProgramUsage->getProgramName();
+	}
+	//-----------------------------------------------------------------------
+	GpuProgramParametersSharedPtr Pass::getGeometryProgramParameters(void) const
+	{
+		OGRE_LOCK_MUTEX(mGpuProgramChangeMutex)
+		return mGeometryProgramUsage->getParameters();
+	}
+	//-----------------------------------------------------------------------
+	const GpuProgramPtr& Pass::getGeometryProgram(void) const
+	{
+		OGRE_LOCK_MUTEX(mGpuProgramChangeMutex)
+		return mGeometryProgramUsage->getProgram();
+	}
+	//-----------------------------------------------------------------------
     bool Pass::isLoaded(void) const
     {
         return mParent->isLoaded();
@@ -1171,6 +1406,8 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     void Pass::setTextureFiltering(TextureFilterOptions filterType)
     {
+		OGRE_LOCK_MUTEX(mTexUnitChangeMutex)
+
         TextureUnitStates::iterator i, iend;
         iend = mTextureUnitStates.end();
         for (i = mTextureUnitStates.begin(); i != iend; ++i)
@@ -1181,6 +1418,7 @@ namespace Ogre {
     // --------------------------------------------------------------------
     void Pass::setTextureAnisotropy(unsigned int maxAniso)
     {
+		OGRE_LOCK_MUTEX(mTexUnitChangeMutex)
         TextureUnitStates::iterator i, iend;
         iend = mTextureUnitStates.end();
         for (i = mTextureUnitStates.begin(); i != iend; ++i)
@@ -1189,7 +1427,7 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
-    void Pass::_updateAutoParamsNoLights(const AutoParamDataSource& source) const
+    void Pass::_updateAutoParamsNoLights(const AutoParamDataSource* source) const
     {
         if (hasVertexProgram())
         {
@@ -1197,6 +1435,12 @@ namespace Ogre {
             mVertexProgramUsage->getParameters()->_updateAutoParamsNoLights(source);
         }
 
+        if (hasGeometryProgram())
+        {
+            // Update geometry program auto params
+            mGeometryProgramUsage->getParameters()->_updateAutoParamsNoLights(source);
+        }
+
         if (hasFragmentProgram())
         {
             // Update fragment program auto params
@@ -1204,7 +1448,7 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
-    void Pass::_updateAutoParamsLightsOnly(const AutoParamDataSource& source) const
+    void Pass::_updateAutoParamsLightsOnly(const AutoParamDataSource* source) const
     {
         if (hasVertexProgram())
         {
@@ -1212,6 +1456,12 @@ namespace Ogre {
             mVertexProgramUsage->getParameters()->_updateAutoParamsLightsOnly(source);
         }
 
+        if (hasGeometryProgram())
+        {
+            // Update geometry program auto params
+            mGeometryProgramUsage->getParameters()->_updateAutoParamsLightsOnly(source);
+        }
+
         if (hasFragmentProgram())
         {
             // Update fragment program auto params
@@ -1228,24 +1478,23 @@ namespace Ogre {
 			iend = msPassGraveyard.end();
 			for (i = msPassGraveyard.begin(); i != iend; ++i)
 			{
-				delete *i;
+				OGRE_DELETE *i;
 			}
 			msPassGraveyard.clear();
 		}
-
+        PassSet tempDirtyHashList;
 		{
 			OGRE_LOCK_MUTEX(msDirtyHashListMutex)
-			PassSet::iterator i, iend;
 			// The dirty ones will have been removed from the groups above using the old hash now
-			iend = msDirtyHashList.end();
-			for (i = msDirtyHashList.begin(); i != iend; ++i)
-			{
-				Pass* p = *i;
-				p->_recalculateHash();
-			}
-			// Clear the dirty list
-			msDirtyHashList.clear();
-		}
+            tempDirtyHashList.swap(msDirtyHashList);
+        }
+        PassSet::iterator i, iend;
+        iend = tempDirtyHashList.end();
+        for (i = tempDirtyHashList.begin(); i != iend; ++i)
+        {
+            Pass* p = *i;
+            p->_recalculateHash();
+        }
     }
     //-----------------------------------------------------------------------
     void Pass::queueForDeletion(void)
@@ -1255,27 +1504,32 @@ namespace Ogre {
         removeAllTextureUnitStates();
         if (mVertexProgramUsage)
         {
-            delete mVertexProgramUsage;
+            OGRE_DELETE mVertexProgramUsage;
             mVertexProgramUsage = 0;
         }
         if (mShadowCasterVertexProgramUsage)
         {
-            delete mShadowCasterVertexProgramUsage;
+            OGRE_DELETE mShadowCasterVertexProgramUsage;
             mShadowCasterVertexProgramUsage = 0;
         }
         if (mShadowReceiverVertexProgramUsage)
         {
-            delete mShadowReceiverVertexProgramUsage;
+            OGRE_DELETE mShadowReceiverVertexProgramUsage;
             mShadowReceiverVertexProgramUsage = 0;
         }
+        if (mGeometryProgramUsage)
+        {
+            delete mGeometryProgramUsage;
+            mGeometryProgramUsage = 0;
+        }
         if (mFragmentProgramUsage)
         {
-            delete mFragmentProgramUsage;
+            OGRE_DELETE mFragmentProgramUsage;
             mFragmentProgramUsage = 0;
         }
 		if (mShadowReceiverFragmentProgramUsage)
 		{
-			delete mShadowReceiverFragmentProgramUsage;
+			OGRE_DELETE mShadowReceiverFragmentProgramUsage;
 			mShadowReceiverFragmentProgramUsage = 0;
 		}
         // remove from dirty list, if there
@@ -1307,14 +1561,14 @@ namespace Ogre {
         // Turn off vertex program if name blank
         if (name.empty())
         {
-            if (mShadowCasterVertexProgramUsage) delete mShadowCasterVertexProgramUsage;
+            if (mShadowCasterVertexProgramUsage) OGRE_DELETE mShadowCasterVertexProgramUsage;
             mShadowCasterVertexProgramUsage = NULL;
         }
         else
         {
             if (!mShadowCasterVertexProgramUsage)
             {
-                mShadowCasterVertexProgramUsage = new GpuProgramUsage(GPT_VERTEX_PROGRAM);
+                mShadowCasterVertexProgramUsage = OGRE_NEW GpuProgramUsage(GPT_VERTEX_PROGRAM);
             }
             mShadowCasterVertexProgramUsage->setProgramName(name);
         }
@@ -1362,14 +1616,14 @@ namespace Ogre {
         // Turn off vertex program if name blank
         if (name.empty())
         {
-            if (mShadowReceiverVertexProgramUsage) delete mShadowReceiverVertexProgramUsage;
+            if (mShadowReceiverVertexProgramUsage) OGRE_DELETE mShadowReceiverVertexProgramUsage;
             mShadowReceiverVertexProgramUsage = NULL;
         }
         else
         {
             if (!mShadowReceiverVertexProgramUsage)
             {
-                mShadowReceiverVertexProgramUsage = new GpuProgramUsage(GPT_VERTEX_PROGRAM);
+                mShadowReceiverVertexProgramUsage = OGRE_NEW GpuProgramUsage(GPT_VERTEX_PROGRAM);
             }
             mShadowReceiverVertexProgramUsage->setProgramName(name);
         }
@@ -1417,14 +1671,14 @@ namespace Ogre {
 		// Turn off Fragment program if name blank
 		if (name.empty())
 		{
-			if (mShadowReceiverFragmentProgramUsage) delete mShadowReceiverFragmentProgramUsage;
+			if (mShadowReceiverFragmentProgramUsage) OGRE_DELETE mShadowReceiverFragmentProgramUsage;
 			mShadowReceiverFragmentProgramUsage = NULL;
 		}
 		else
 		{
 			if (!mShadowReceiverFragmentProgramUsage)
 			{
-				mShadowReceiverFragmentProgramUsage = new GpuProgramUsage(GPT_FRAGMENT_PROGRAM);
+				mShadowReceiverFragmentProgramUsage = OGRE_NEW GpuProgramUsage(GPT_FRAGMENT_PROGRAM);
 			}
 			mShadowReceiverFragmentProgramUsage->setProgramName(name);
 		}
