@@ -41,6 +41,7 @@ Torus Knot Software Ltd.
 #include "OgreDynLib.h"
 #include "OgreConfigFile.h"
 #include "OgreMaterialManager.h"
+#include "OgreRenderSystemCapabilitiesManager.h"
 #include "OgreMeshManager.h"
 #include "OgreTextureManager.h"
 #include "OgreParticleSystemManager.h"
@@ -86,6 +87,8 @@ Torus Knot Software Ltd.
 #include "OgreExternalTextureSourceManager.h"
 #include "OgreCompositorManager.h"
 
+#include "OgreScriptCompiler.h"
+
 #include "OgreWindowEventUtilities.h"
 
 namespace Ogre {
@@ -107,8 +110,12 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     Root::Root(const String& pluginFileName, const String& configFileName, 
 		const String& logFileName)
-      : mLogManager(0), mCurrentFrame(0), mFrameSmoothingTime(0.0f),
-	  mNextMovableObjectTypeFlag(1), mIsInitialised(false)
+      : mLogManager(0)
+	  , mRenderSystemCapabilitiesManager(0)
+	  , mNextFrame(0)
+	  , mFrameSmoothingTime(0.0f)
+	  , mNextMovableObjectTypeFlag(1)
+	  , mIsInitialised(false)
     {
         // superclass will do singleton checking
         String msg;
@@ -125,63 +132,68 @@ namespace Ogre {
 		// Create log manager and default log file if there is no log manager yet
 		if(LogManager::getSingletonPtr() == 0)
 		{
-			mLogManager = new LogManager();
+			mLogManager = OGRE_NEW LogManager();
 			mLogManager->createLog(logFileName, true, true);
 		}
 
         // Dynamic library manager
-        mDynLibManager = new DynLibManager();
+        mDynLibManager = OGRE_NEW DynLibManager();
 
-        mArchiveManager = new ArchiveManager();
+        mArchiveManager = OGRE_NEW ArchiveManager();
 
 		// ResourceGroupManager
-		mResourceGroupManager = new ResourceGroupManager();
+		mResourceGroupManager = OGRE_NEW ResourceGroupManager();
 
 		// ResourceBackgroundQueue
-		mResourceBackgroundQueue = new ResourceBackgroundQueue();
+		mResourceBackgroundQueue = OGRE_NEW ResourceBackgroundQueue();
 
 		// Create SceneManager enumerator (note - will be managed by singleton)
-        mSceneManagerEnum = new SceneManagerEnumerator();
+        mSceneManagerEnum = OGRE_NEW SceneManagerEnumerator();
         mCurrentSceneManager = NULL;
 
-		mShadowTextureManager = new ShadowTextureManager();
+		mShadowTextureManager = OGRE_NEW ShadowTextureManager();
+
+		mRenderSystemCapabilitiesManager = OGRE_NEW RenderSystemCapabilitiesManager();
 
         // ..material manager
-        mMaterialManager = new MaterialManager();
+        mMaterialManager = OGRE_NEW MaterialManager();
 
         // Mesh manager
-        mMeshManager = new MeshManager();
+        mMeshManager = OGRE_NEW MeshManager();
 
         // Skeleton manager
-        mSkeletonManager = new SkeletonManager();
+        mSkeletonManager = OGRE_NEW SkeletonManager();
 
         // ..particle system manager
-        mParticleManager = new ParticleSystemManager();
+        mParticleManager = OGRE_NEW ParticleSystemManager();
 
-        mTimer = new Timer();
+		// Compiler manager
+		//mCompilerManager = OGRE_NEW ScriptCompilerManager();
+
+        mTimer = OGRE_NEW Timer();
 
         // Overlay manager
-        mOverlayManager = new OverlayManager();
+        mOverlayManager = OGRE_NEW OverlayManager();
 
-        mPanelFactory = new PanelOverlayElementFactory();
+        mPanelFactory = OGRE_NEW PanelOverlayElementFactory();
         mOverlayManager->addOverlayElementFactory(mPanelFactory);
 
-        mBorderPanelFactory = new BorderPanelOverlayElementFactory();
+        mBorderPanelFactory = OGRE_NEW BorderPanelOverlayElementFactory();
         mOverlayManager->addOverlayElementFactory(mBorderPanelFactory);
 
-        mTextAreaFactory = new TextAreaOverlayElementFactory();
+        mTextAreaFactory = OGRE_NEW TextAreaOverlayElementFactory();
         mOverlayManager->addOverlayElementFactory(mTextAreaFactory);
         // Font manager
-        mFontManager = new FontManager();
+        mFontManager = OGRE_NEW FontManager();
 
 #if OGRE_PROFILING
         // Profiler
-        mProfiler = new Profiler();
+        mProfiler = OGRE_NEW Profiler();
 		Profiler::getSingleton().setTimer(mTimer);
 #endif
-        mFileSystemArchiveFactory = new FileSystemArchiveFactory();
+        mFileSystemArchiveFactory = OGRE_NEW FileSystemArchiveFactory();
         ArchiveManager::getSingleton().addArchiveFactory( mFileSystemArchiveFactory );
-        mZipArchiveFactory = new ZipArchiveFactory();
+        mZipArchiveFactory = OGRE_NEW ZipArchiveFactory();
         ArchiveManager::getSingleton().addArchiveFactory( mZipArchiveFactory );
 #if OGRE_NO_FREEIMAGE == 0
 		// Register image codecs
@@ -196,26 +208,28 @@ namespace Ogre {
 		DDSCodec::startup();
 #endif
 
-        mHighLevelGpuProgramManager = new HighLevelGpuProgramManager();
+        mHighLevelGpuProgramManager = OGRE_NEW HighLevelGpuProgramManager();
 
-		mExternalTextureSourceManager = new ExternalTextureSourceManager();
-        mCompositorManager = new CompositorManager();
+		mExternalTextureSourceManager = OGRE_NEW ExternalTextureSourceManager();
+        mCompositorManager = OGRE_NEW CompositorManager();
+
+		mCompilerManager = OGRE_NEW ScriptCompilerManager();
 
         // Auto window
         mAutoWindow = 0;
 
 		// instantiate and register base movable factories
-		mEntityFactory = new EntityFactory();
+		mEntityFactory = OGRE_NEW EntityFactory();
 		addMovableObjectFactory(mEntityFactory);
-		mLightFactory = new LightFactory();
+		mLightFactory = OGRE_NEW LightFactory();
 		addMovableObjectFactory(mLightFactory);
-		mBillboardSetFactory = new BillboardSetFactory();
+		mBillboardSetFactory = OGRE_NEW BillboardSetFactory();
 		addMovableObjectFactory(mBillboardSetFactory);
-		mManualObjectFactory = new ManualObjectFactory();
+		mManualObjectFactory = OGRE_NEW ManualObjectFactory();
 		addMovableObjectFactory(mManualObjectFactory);
-		mBillboardChainFactory = new BillboardChainFactory();
+		mBillboardChainFactory = OGRE_NEW BillboardChainFactory();
 		addMovableObjectFactory(mBillboardChainFactory);
-		mRibbonTrailFactory = new RibbonTrailFactory();
+		mRibbonTrailFactory = OGRE_NEW RibbonTrailFactory();
 		addMovableObjectFactory(mRibbonTrailFactory);
 
 		// Load plugins
@@ -237,12 +251,13 @@ namespace Ogre {
     Root::~Root()
     {
         shutdown();
-        delete mSceneManagerEnum;
-		delete mShadowTextureManager;
+        OGRE_DELETE mSceneManagerEnum;
+		OGRE_DELETE mShadowTextureManager;
+		OGRE_DELETE mRenderSystemCapabilitiesManager;
 
 		destroyAllRenderQueueInvocationSequences();
-        delete mCompositorManager;
-		delete mExternalTextureSourceManager;
+        OGRE_DELETE mCompositorManager;
+		OGRE_DELETE mExternalTextureSourceManager;
 #if OGRE_NO_FREEIMAGE == 0
 		FreeImageCodec::shutdown();
 #endif
@@ -253,43 +268,46 @@ namespace Ogre {
 		DDSCodec::shutdown();
 #endif
 #if OGRE_PROFILING
-        delete mProfiler;
+        OGRE_DELETE mProfiler;
 #endif
-        delete mOverlayManager;
-        delete mFontManager;
-        delete mArchiveManager;
-        delete mZipArchiveFactory;
-        delete mFileSystemArchiveFactory;
-        delete mSkeletonManager;
-        delete mMeshManager;
-        delete mParticleManager;
+        OGRE_DELETE mOverlayManager;
+        OGRE_DELETE mFontManager;
+        OGRE_DELETE mArchiveManager;
+        OGRE_DELETE mZipArchiveFactory;
+        OGRE_DELETE mFileSystemArchiveFactory;
+        OGRE_DELETE mSkeletonManager;
+        OGRE_DELETE mMeshManager;
+        OGRE_DELETE mParticleManager;
+		//OGRE_DELETE mCompilerManager;
 
         if( mControllerManager )
-            delete mControllerManager;
+            OGRE_DELETE mControllerManager;
         if (mHighLevelGpuProgramManager)
-            delete mHighLevelGpuProgramManager;
+            OGRE_DELETE mHighLevelGpuProgramManager;
 
-        delete mTextAreaFactory;
-        delete mBorderPanelFactory;
-        delete mPanelFactory;
+        OGRE_DELETE mTextAreaFactory;
+        OGRE_DELETE mBorderPanelFactory;
+        OGRE_DELETE mPanelFactory;
 
         unloadPlugins();
-        delete mMaterialManager;
+        OGRE_DELETE mMaterialManager;
         Pass::processPendingPassUpdates(); // make sure passes are cleaned
-		delete mResourceBackgroundQueue;
-        delete mResourceGroupManager;
+		OGRE_DELETE mResourceBackgroundQueue;
+        OGRE_DELETE mResourceGroupManager;
 
-		delete mEntityFactory;
-		delete mLightFactory;
-		delete mBillboardSetFactory;
-		delete mManualObjectFactory;
-		delete mBillboardChainFactory;
-		delete mRibbonTrailFactory;
+		OGRE_DELETE mEntityFactory;
+		OGRE_DELETE mLightFactory;
+		OGRE_DELETE mBillboardSetFactory;
+		OGRE_DELETE mManualObjectFactory;
+		OGRE_DELETE mBillboardChainFactory;
+		OGRE_DELETE mRibbonTrailFactory;
 
-	delete mTimer;
+		OGRE_DELETE mTimer;
 
-        delete mDynLibManager;
-        delete mLogManager;
+        OGRE_DELETE mDynLibManager;
+        OGRE_DELETE mLogManager;
+
+		OGRE_DELETE mCompilerManager;
 
 		mAutoWindow = 0;
 		mFirstTimePostWindowInit = false;
@@ -407,12 +425,12 @@ namespace Ogre {
 
         restoreConfig();
 
-        dlg = new ConfigDialog();
+        dlg = OGRE_NEW ConfigDialog();
 
         if ((isOk = dlg->display()))
             saveConfig();
 
-        delete dlg;
+        OGRE_DELETE dlg;
         return isOk;
     }
 
@@ -484,7 +502,7 @@ namespace Ogre {
     }
 
     //-----------------------------------------------------------------------
-    RenderWindow* Root::initialise(bool autoCreateWindow, const String& windowTitle)
+    RenderWindow* Root::initialise(bool autoCreateWindow, const String& windowTitle, const String& customCapabilitiesConfig)
     {
         if (!mActiveRenderer)
             OGRE_EXCEPT(Exception::ERR_INVALID_STATE,
@@ -492,10 +510,44 @@ namespace Ogre {
             "system has been selected.", "Root::initialise");
 
         if (!mControllerManager)
-			mControllerManager = new ControllerManager();
+			mControllerManager = OGRE_NEW ControllerManager();
+
+        // .rendercaps manager
+        RenderSystemCapabilitiesManager& rscManager = RenderSystemCapabilitiesManager::getSingleton();
+        // caller wants to load custom RenderSystemCapabilities form a config file
+        if(customCapabilitiesConfig != StringUtil::BLANK)
+        {
+            ConfigFile cfg;
+            cfg.load(customCapabilitiesConfig, "\t:=", false);
+
+            // Capabilities Database setting must be in the same format as
+            // resources.cfg in Ogre examples.
+            ConfigFile::SettingsIterator iter = cfg.getSettingsIterator("Capabilities Database");
+            while(iter.hasMoreElements())
+            {
+                String archType = iter.peekNextKey();
+                String filename = iter.getNext();
+
+                rscManager.parseCapabilitiesFromArchive(filename, archType, true);
+            }
+
+            String capsName = cfg.getSetting("Custom Capabilities");
+            // The custom capabilities have been parsed, let's retrieve them
+            RenderSystemCapabilities* rsc = rscManager.loadParsedCapabilities(capsName);
+			if(rsc == 0)
+			{
+				OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND,
+					String("Cannot load a RenderSystemCapability named ") + capsName,
+					"Root::initialise");
+			}
+
+            // Tell RenderSystem to use the comon rsc
+            useCustomRenderSystemCapabilities(rsc);
+        }
+
 
 		PlatformInformation::log(LogManager::getSingleton().getDefaultLog());
-		mAutoWindow =  mActiveRenderer->initialise(autoCreateWindow, windowTitle);
+		mAutoWindow =  mActiveRenderer->_initialise(autoCreateWindow, windowTitle);
 
 
         if (autoCreateWindow && !mFirstTimePostWindowInit)
@@ -516,6 +568,11 @@ namespace Ogre {
 
     }
     //-----------------------------------------------------------------------
+    void Root::useCustomRenderSystemCapabilities(RenderSystemCapabilities* capabilities)
+    {
+        mActiveRenderer->useCustomRenderSystemCapabilities(capabilities);
+    }
+    //-----------------------------------------------------------------------
     String Root::getErrorDescription(long errorNumber)
     {
 
@@ -607,9 +664,6 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     bool Root::_fireFrameStarted(FrameEvent& evt)
     {
-        // Increment frame number
-        ++mCurrentFrame;
-
         // Remove all marked listeners
         std::set<FrameListener*>::iterator i;
         for (i = mRemovedFrameListeners.begin();
@@ -628,6 +682,31 @@ namespace Ogre {
 
         return true;
 
+    }
+	//-----------------------------------------------------------------------
+    bool Root::_fireFrameRenderingQueued(FrameEvent& evt)
+    {
+		// Increment next frame number
+		++mNextFrame;
+
+        // Remove all marked listeners
+        std::set<FrameListener*>::iterator i;
+        for (i = mRemovedFrameListeners.begin();
+            i != mRemovedFrameListeners.end(); i++)
+        {
+            mFrameListeners.erase(*i);
+        }
+        mRemovedFrameListeners.clear();
+
+        // Tell all listeners
+        for (i= mFrameListeners.begin(); i != mFrameListeners.end(); ++i)
+        {
+            if (!(*i)->frameRenderingQueued(evt))
+                return false;
+        }
+
+        return true;
+
     }
     //-----------------------------------------------------------------------
     bool Root::_fireFrameEnded(FrameEvent& evt)
@@ -657,7 +736,7 @@ namespace Ogre {
             HardwareBufferManager::getSingleton()._releaseBufferCopies();
 
 		// Also tell the ResourceBackgroundQueue to propagate background load events
-		ResourceBackgroundQueue::getSingleton()._fireBackgroundLoadingComplete();
+		ResourceBackgroundQueue::getSingleton()._fireOnFrameCallbacks();
 
         return ret;
     }
@@ -672,6 +751,16 @@ namespace Ogre {
         return _fireFrameStarted(evt);
     }
     //-----------------------------------------------------------------------
+    bool Root::_fireFrameRenderingQueued()
+    {
+        unsigned long now = mTimer->getMilliseconds();
+        FrameEvent evt;
+        evt.timeSinceLastEvent = calculateEventTime(now, FETT_ANY);
+        evt.timeSinceLastFrame = calculateEventTime(now, FETT_QUEUED);
+
+        return _fireFrameRenderingQueued(evt);
+    }
+    //-----------------------------------------------------------------------
     bool Root::_fireFrameEnded()
     {
         unsigned long now = mTimer->getMilliseconds();
@@ -725,6 +814,7 @@ namespace Ogre {
 
         mActiveRenderer->_initRenderTargets();
 
+        // Clear event times
 		clearEventTimes();
 
         // Infinite loop, until broken out of by frame listeners
@@ -746,7 +836,8 @@ namespace Ogre {
         if(!_fireFrameStarted())
             return false;
 
-        _updateAllRenderTargets();
+		if (!_updateAllRenderTargets())
+			return false;
 
         return _fireFrameEnded();
     }
@@ -788,6 +879,14 @@ namespace Ogre {
         pluginDir = cfg.getSetting("PluginFolder"); // Ignored on Mac OS X, uses Resources/ directory
         pluginList = cfg.getMultiSetting("Plugin");
 
+#if OGRE_PLATFORM != OGRE_PLATFORM_APPLE
+		if (pluginDir.empty())
+		{
+			// User didn't specify plugins folder, try current one
+			pluginDir = ".";
+		}
+#endif
+
         char last_char = pluginDir[pluginDir.length()-1];
         if (last_char != '/' && last_char != '\\')
         {
@@ -882,7 +981,7 @@ namespace Ogre {
             "system has been selected.", "Root::createRenderWindow");
         }
         RenderWindow* ret;
-        ret = mActiveRenderer->createRenderWindow(name, width, height, fullScreen, miscParams);
+        ret = mActiveRenderer->_createRenderWindow(name, width, height, fullScreen, miscParams);
 
         // Initialisation for classes dependent on first window created
         if(!mFirstTimePostWindowInit)
@@ -1027,16 +1126,22 @@ namespace Ogre {
 
     }
     //-----------------------------------------------------------------------
-    void Root::_updateAllRenderTargets(void)
+    bool Root::_updateAllRenderTargets(void)
     {
-        // delegate
-        mActiveRenderer->_updateAllRenderTargets();
-    }
+        // update all targets but don't swap buffers
+        mActiveRenderer->_updateAllRenderTargets(false);
+		// give client app opportunity to use queued GPU time
+		bool ret = _fireFrameRenderingQueued();
+		// block for final swap
+		mActiveRenderer->_swapAllRenderTargetBuffers(mActiveRenderer->getWaitForVerticalBlank());
+		
+		return ret;
+	}
 	//-----------------------------------------------------------------------
 	void Root::clearEventTimes(void)
 	{
 		// Clear event times
-		for(int i=0; i<3; ++i)
+		for(int i=0; i<FETT_COUNT; ++i)
 			mEventTimes[i].clear();
 	}
 	//---------------------------------------------------------------------
@@ -1139,7 +1244,7 @@ namespace Ogre {
 					" already exists.",
 				"Root::createRenderQueueInvocationSequence");
 		}
-		RenderQueueInvocationSequence* ret = new RenderQueueInvocationSequence(name);
+		RenderQueueInvocationSequence* ret = OGRE_NEW RenderQueueInvocationSequence(name);
 		mRQSequenceMap[name] = ret;
 		return ret;
 	}
@@ -1166,7 +1271,7 @@ namespace Ogre {
 			mRQSequenceMap.find(name);
 		if (i != mRQSequenceMap.end())
 		{
-			delete i->second;
+			OGRE_DELETE i->second;
 			mRQSequenceMap.erase(i);
 		}
 	}
@@ -1176,7 +1281,7 @@ namespace Ogre {
 		for (RenderQueueInvocationSequenceMap::iterator i = mRQSequenceMap.begin();
 			i != mRQSequenceMap.end(); ++i)
 		{
-			delete i->second;
+			OGRE_DELETE i->second;
 		}
 		mRQSequenceMap.clear();
 	}
