@@ -78,7 +78,7 @@ namespace Ogre {
         mObjectsByName.clear();
 
         if (mWireBoundingBox) {
-			delete mWireBoundingBox;
+			OGRE_DELETE mWireBoundingBox;
 		}
     }
     //-----------------------------------------------------------------------
@@ -302,7 +302,8 @@ namespace Ogre {
 				if (visibleBounds)
 				{
 					visibleBounds->merge(iobj->second->getWorldBoundingBox(true), 
-						iobj->second->getWorldBoundingSphere(true), cam);
+						iobj->second->getWorldBoundingSphere(true), cam, 
+						queue->getQueueGroup(iobj->second->getRenderQueueGroup())->getShadowsEnabled());
 				}
             }
         }
@@ -339,7 +340,7 @@ namespace Ogre {
 	void SceneNode::_addBoundingBoxToQueue(RenderQueue* queue) {
 		// Create a WireBoundingBox if needed.
 		if (mWireBoundingBox == NULL) {
-			mWireBoundingBox = new WireBoundingBox();
+			mWireBoundingBox = OGRE_NEW WireBoundingBox();
 		}
 		mWireBoundingBox->setupBoundingBox(mWorldAABB);
 		queue->addRenderable(mWireBoundingBox);
@@ -648,6 +649,26 @@ namespace Ogre {
             }
         }
     }
+	//-----------------------------------------------------------------------
+	void SceneNode::setDebugDisplayEnabled(bool enabled, bool cascade)
+	{
+		ObjectMap::iterator oi, oiend;
+		oiend = mObjectsByName.end();
+		for (oi = mObjectsByName.begin(); oi != oiend; ++oi)
+		{
+			oi->second->setDebugDisplayEnabled(enabled);
+		}
+
+		if (cascade)
+		{
+			ChildNodeMap::iterator i, iend;
+			iend = mChildren.end();
+			for (i = mChildren.begin(); i != iend; ++i)
+			{
+				static_cast<SceneNode*>(i->second)->setDebugDisplayEnabled(enabled, cascade);
+			}
+		}
+	}
     //-----------------------------------------------------------------------
     void SceneNode::flipVisibility(bool cascade)
     {
